from _typeshed import Incomplete
from collections.abc import Generator, Iterable, ValuesView
from typing import Optional
from rdkit import Chem as Chem
from rdkit.Chem import Mol

environs: dict[str, str]
reactionDefs: list[tuple[str, str, str]]
smartsGps: Incomplete
environMatchers: dict[str, Mol]
bondMatchers: Incomplete
reactions: Incomplete
reverseReactions: Incomplete
rs: Incomplete
ps: Incomplete
rxn: Incomplete
labels: Incomplete

def FindBRICSBonds(
    mol: Mol, randomizeOrder: bool = False, silent: bool = True
) -> Generator[tuple[tuple[int, int], tuple[str, str]], None, None]:
    """
    returns the bonds in a molecule that BRICS would cleave

    >>> from rdkit import Chem
    >>> m = Chem.MolFromSmiles('CCCOCC')
    >>> res = list(FindBRICSBonds(m))
    >>> res
    [((3, 2), ('3', '4')), ((3, 4), ('3', '4'))]

    a more complicated case:

    >>> m = Chem.MolFromSmiles('CCCOCCC(=O)c1ccccc1')
    >>> res = list(FindBRICSBonds(m))
    >>> res
    [((3, 2), ('3', '4')), ((3, 4), ('3', '4')), ((6, 8), ('6', '16'))]

    we can also randomize the order of the results:

    >>> random.seed(23)
    >>> res = list(FindBRICSBonds(m,randomizeOrder=True))
    >>> sorted(res)
    [((3, 2), ('3', '4')), ((3, 4), ('3', '4')), ((6, 8), ('6', '16'))]

    Note that this is a generator function :

    >>> res = FindBRICSBonds(m)
    >>> res
    <generator object ...>
    >>> next(res)
    ((3, 2), ('3', '4'))

    >>> m = Chem.MolFromSmiles('CC=CC')
    >>> res = list(FindBRICSBonds(m))
    >>> sorted(res)
    [((1, 2), ('7', '7'))]

    make sure we don't match ring bonds:

    >>> m = Chem.MolFromSmiles('O=C1NCCC1')
    >>> list(FindBRICSBonds(m))
    []

    another nice one, make sure environment 8 doesn't match something connected
    to a ring atom:

    >>> m = Chem.MolFromSmiles('CC1(C)CCCCC1')
    >>> list(FindBRICSBonds(m))
    []

    """

def BreakBRICSBonds(
    mol: Mol,
    bonds: Optional[list[tuple[tuple[int, int], tuple[str, str]]]] = None,
    sanitize: bool = True,
    silent: bool = True,
) -> Mol:
    """
    breaks the BRICS bonds in a molecule and returns the results

    >>> from rdkit import Chem
    >>> m = Chem.MolFromSmiles('CCCOCC')
    >>> m2=BreakBRICSBonds(m)
    >>> Chem.MolToSmiles(m2,True)
    '[3*]O[3*].[4*]CC.[4*]CCC'

    a more complicated case:

    >>> m = Chem.MolFromSmiles('CCCOCCC(=O)c1ccccc1')
    >>> m2=BreakBRICSBonds(m)
    >>> Chem.MolToSmiles(m2,True)
    '[16*]c1ccccc1.[3*]O[3*].[4*]CCC.[4*]CCC([6*])=O'


    can also specify a limited set of bonds to work with:

    >>> m = Chem.MolFromSmiles('CCCOCC')
    >>> m2 = BreakBRICSBonds(m,[((3, 2), ('3', '4'))])
    >>> Chem.MolToSmiles(m2,True)
    '[3*]OCC.[4*]CCC'

    this can be used as an alternate approach for doing a BRICS decomposition by
    following BreakBRICSBonds with a call to Chem.GetMolFrags:

    >>> m = Chem.MolFromSmiles('CCCOCC')
    >>> m2=BreakBRICSBonds(m)
    >>> frags = Chem.GetMolFrags(m2,asMols=True)
    >>> [Chem.MolToSmiles(x,True) for x in frags]
    ['[4*]CCC', '[3*]O[3*]', '[4*]CC']

    """

def BRICSDecompose(
    mol: Mol,
    allNodes: Optional[set[str]] = None,
    minFragmentSize: int = 1,
    onlyUseReactions: Optional[tuple] = None,
    silent: bool = True,
    keepNonLeafNodes: bool = False,
    singlePass: bool = False,
    returnMols: bool = False,
) -> set[str] | ValuesView:
    """
    returns the BRICS decomposition for a molecule

    >>> from rdkit import Chem
    >>> m = Chem.MolFromSmiles('CCCOCc1cc(c2ncccc2)ccc1')
    >>> res = list(BRICSDecompose(m))
    >>> sorted(res)
    ['[14*]c1ccccn1', '[16*]c1cccc([16*])c1', '[3*]O[3*]', '[4*]CCC', '[4*]C[8*]']

    >>> res = list(BRICSDecompose(m,returnMols=True))
    >>> res[0]
    <rdkit.Chem.rdchem.Mol object ...>
    >>> smis = [Chem.MolToSmiles(x,True) for x in res]
    >>> sorted(smis)
    ['[14*]c1ccccn1', '[16*]c1cccc([16*])c1', '[3*]O[3*]', '[4*]CCC', '[4*]C[8*]']

    nexavar, an example from the paper (corrected):

    >>> m = Chem.MolFromSmiles('CNC(=O)C1=NC=CC(OC2=CC=C(NC(=O)NC3=CC(=C(Cl)C=C3)C(F)(F)F)C=C2)=C1')
    >>> res = list(BRICSDecompose(m))
    >>> sorted(res)
    ['[1*]C([1*])=O', '[1*]C([6*])=O', '[14*]c1cc([16*])ccn1', '[16*]c1ccc(Cl)c([16*])c1', '[16*]c1ccc([16*])cc1', '[3*]O[3*]', '[5*]NC', '[5*]N[5*]', '[8*]C(F)(F)F']

    it's also possible to keep pieces that haven't been fully decomposed:

    >>> m = Chem.MolFromSmiles('CCCOCC')
    >>> res = list(BRICSDecompose(m,keepNonLeafNodes=True))
    >>> sorted(res)
    ['CCCOCC', '[3*]OCC', '[3*]OCCC', '[3*]O[3*]', '[4*]CC', '[4*]CCC']

    >>> m = Chem.MolFromSmiles('CCCOCc1cc(c2ncccc2)ccc1')
    >>> res = list(BRICSDecompose(m,keepNonLeafNodes=True))
    >>> sorted(res)
    ['CCCOCc1cccc(-c2ccccn2)c1', '[14*]c1ccccn1', '[16*]c1cccc(-c2ccccn2)c1', '[16*]c1cccc(COCCC)c1', '[16*]c1cccc([16*])c1', '[3*]OCCC', '[3*]OC[8*]', '[3*]OCc1cccc(-c2ccccn2)c1', '[3*]OCc1cccc([16*])c1', '[3*]O[3*]', '[4*]CCC', '[4*]C[8*]', '[4*]Cc1cccc(-c2ccccn2)c1', '[4*]Cc1cccc([16*])c1', '[8*]COCCC']

    or to only do a single pass of decomposition:

    >>> m = Chem.MolFromSmiles('CCCOCc1cc(c2ncccc2)ccc1')
    >>> res = list(BRICSDecompose(m,singlePass=True))
    >>> sorted(res)
    ['CCCOCc1cccc(-c2ccccn2)c1', '[14*]c1ccccn1', '[16*]c1cccc(-c2ccccn2)c1', '[16*]c1cccc(COCCC)c1', '[3*]OCCC', '[3*]OCc1cccc(-c2ccccn2)c1', '[4*]CCC', '[4*]Cc1cccc(-c2ccccn2)c1', '[8*]COCCC']

    setting a minimum size for the fragments:

    >>> m = Chem.MolFromSmiles('CCCOCC')
    >>> res = list(BRICSDecompose(m,keepNonLeafNodes=True,minFragmentSize=2))
    >>> sorted(res)
    ['CCCOCC', '[3*]OCC', '[3*]OCCC', '[4*]CC', '[4*]CCC']
    >>> m = Chem.MolFromSmiles('CCCOCC')
    >>> res = list(BRICSDecompose(m,keepNonLeafNodes=True,minFragmentSize=3))
    >>> sorted(res)
    ['CCCOCC', '[3*]OCC', '[4*]CCC']
    >>> res = list(BRICSDecompose(m,minFragmentSize=2))
    >>> sorted(res)
    ['[3*]OCC', '[3*]OCCC', '[4*]CC', '[4*]CCC']


    """

def BRICSBuild(
    fragments,
    onlyCompleteMols: bool = True,
    seeds: Optional[Iterable] = None,
    uniquify: bool = True,
    scrambleReagents: bool = True,
    maxDepth: int = 3,
) -> Generator[Mol, None, None]: ...
def _test(): ...



from _typeshed import Incomplete

mcimport: Incomplete

def _myimport(
    name,
    globals: Incomplete | None = ...,
    locals: Incomplete | None = ...,
    fromlist: Incomplete | None = ...,
): ...
def set_gen_path(path) -> None: ...



from typing import Any, ClassVar

class HashFunction:
    AnonymousGraph: ClassVar[HashFunction] = ...
    ArthorSubstructureOrder: ClassVar[HashFunction] = ...
    AtomBondCounts: ClassVar[HashFunction] = ...
    CanonicalSmiles: ClassVar[HashFunction] = ...
    DegreeVector: ClassVar[HashFunction] = ...
    ElementGraph: ClassVar[HashFunction] = ...
    ExtendedMurcko: ClassVar[HashFunction] = ...
    HetAtomProtomer: ClassVar[HashFunction] = ...
    HetAtomTautomer: ClassVar[HashFunction] = ...
    Mesomer: ClassVar[HashFunction] = ...
    MolFormula: ClassVar[HashFunction] = ...
    MurckoScaffold: ClassVar[HashFunction] = ...
    NetCharge: ClassVar[HashFunction] = ...
    RedoxPair: ClassVar[HashFunction] = ...
    Regioisomer: ClassVar[HashFunction] = ...
    SmallWorldIndexBR: ClassVar[HashFunction] = ...
    SmallWorldIndexBRL: ClassVar[HashFunction] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

def MolHash(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem, RDConfig as RDConfig

defaultPatternFileName: Incomplete

def _CountMatches(mol, patt, unique: bool = ...): ...

fns: Incomplete

def _LoadPatterns(fileName: Incomplete | None = ...): ...

fn: Incomplete



from typing import Any, ClassVar

class MHFPEncoder:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def CreateShinglingFromMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def CreateShinglingFromSmiles(cls, *args, **kwargs) -> Any: ...
    def Distance(self, *args, **kwargs) -> Any: ...
    @classmethod
    def EncodeMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def EncodeMolsBulk(cls, RDKit, boost) -> Any: ...
    @classmethod
    def EncodeSECFPMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def EncodeSECFPMolsBulk(cls, RDKit, boost) -> Any: ...
    @classmethod
    def EncodeSECFPSmiles(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def EncodeSECFPSmilesBulk(cls, RDKit, boost) -> Any: ...
    @classmethod
    def EncodeSmiles(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def EncodeSmilesBulk(cls, RDKit, boost) -> Any: ...
    @classmethod
    def FromArray(cls, RDKit, boost) -> Any: ...
    @classmethod
    def FromStringArray(cls, RDKit, boost) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...



def CollectVotes(composite, data, badOnly): ...
def BuildVoteImage(
    nModels,
    data,
    values,
    trueValues=...,
    sortTrueVals: int = ...,
    xScale: int = ...,
    yScale: int = ...,
    addLine: int = ...,
): ...
def VoteAndBuildImage(
    composite,
    data,
    badOnly: int = ...,
    sortTrueVals: int = ...,
    xScale: int = ...,
    yScale: int = ...,
    addLine: int = ...,
): ...
def Usage() -> None: ...



from _typeshed import Incomplete

class VisOpts:
    circRad: int
    minCircRad: int
    maxCircRad: int
    circColor: Incomplete
    terminalEmptyColor: Incomplete
    terminalOnColor: Incomplete
    terminalOffColor: Incomplete
    outlineColor: Incomplete
    lineColor: Incomplete
    lineWidth: int
    horizOffset: int
    vertOffset: int
    topMargin: int
    labelFont: Incomplete
    highlightColor: Incomplete
    highlightWidth: int

visOpts: Incomplete

def GetMinCanvasSize(adjList, levelList): ...
def DrawHierarchy(
    adjList,
    levelList,
    canvas,
    entryColors: Incomplete | None = ...,
    bitIds: Incomplete | None = ...,
    minLevel: int = ...,
    maxLevel: float = ...,
): ...



from typing import Any, ClassVar

class DeprotectData:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def isValid(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def abbreviation(self) -> Any: ...
    @property
    def deprotection_class(self) -> Any: ...
    @property
    def example(self) -> Any: ...
    @property
    def full_name(self) -> Any: ...
    @property
    def reaction_smarts(self) -> Any: ...

class DeprotectDataVect:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

def Deprotect(RDKit) -> Any: ...
def DeprotectInPlace(RDKit) -> Any: ...
def GetDeprotections() -> Any: ...



from _typeshed import Incomplete

INCHI_AVAILABLE: bool

class InchiReadWriteError(Exception): ...

def MolFromInchi(
    inchi,
    sanitize: bool = ...,
    removeHs: bool = ...,
    logLevel: Incomplete | None = ...,
    treatWarningAsError: bool = ...,
): ...
def MolToInchiAndAuxInfo(
    mol,
    options: str = ...,
    logLevel: Incomplete | None = ...,
    treatWarningAsError: bool = ...,
): ...
def MolBlockToInchiAndAuxInfo(
    molblock,
    options: str = ...,
    logLevel: Incomplete | None = ...,
    treatWarningAsError: bool = ...,
): ...
def MolToInchi(
    mol,
    options: str = ...,
    logLevel: Incomplete | None = ...,
    treatWarningAsError: bool = ...,
): ...
def MolBlockToInchi(
    molblock,
    options: str = ...,
    logLevel: Incomplete | None = ...,
    treatWarningAsError: bool = ...,
): ...
def InchiToInchiKey(inchi): ...
def MolToInchiKey(mol, options: str = ...): ...



from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs
from rdkit.DataStructs import TopNContainer as TopNContainer

class SimilarityScreener:
    metric: Incomplete
    dataSource: Incomplete
    fingerprinter: Incomplete
    probe: Incomplete
    def __init__(
        self,
        probe: Incomplete | None = ...,
        metric: Incomplete | None = ...,
        dataSource: Incomplete | None = ...,
        fingerprinter: Incomplete | None = ...,
    ) -> None: ...
    def Reset(self) -> None: ...
    def SetProbe(self, probeFingerprint) -> None: ...
    def GetSingleFingerprint(self, probe): ...

class ThresholdScreener(SimilarityScreener):
    threshold: Incomplete
    dataIter: Incomplete
    def __init__(self, threshold, **kwargs) -> None: ...
    def _nextMatch(self): ...
    def Reset(self) -> None: ...
    def __iter__(self): ...
    def next(self): ...
    __next__: Incomplete

class TopNScreener(SimilarityScreener):
    numToGet: Incomplete
    topN: Incomplete
    _pos: int
    def __init__(self, num, **kwargs) -> None: ...
    def Reset(self) -> None: ...
    def __iter__(self): ...
    def next(self): ...
    __next__: Incomplete
    def _initTopN(self) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, idx): ...



from typing import Any, ClassVar

class EdgeType:
    Fragment: ClassVar[EdgeType] = ...
    Generic: ClassVar[EdgeType] = ...
    GenericBond: ClassVar[EdgeType] = ...
    Initialize: ClassVar[EdgeType] = ...
    RemoveAttachment: ClassVar[EdgeType] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class NetworkEdge:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def beginIdx(self) -> Any: ...
    @property
    def endIdx(self) -> Any: ...
    @property
    def type(self) -> Any: ...

class NetworkEdge_VECT:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class ScaffoldNetwork:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __getinitargs__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def counts(self) -> Any: ...
    @property
    def edges(self) -> Any: ...
    @property
    def molCounts(self) -> Any: ...
    @property
    def nodes(self) -> Any: ...

class ScaffoldNetworkParams:
    __instance_size__: ClassVar[int] = ...
    collectMolCounts: Any
    flattenChirality: Any
    flattenIsotopes: Any
    flattenKeepLargest: Any
    includeGenericBondScaffolds: Any
    includeGenericScaffolds: Any
    includeScaffoldsWithAttachments: Any
    includeScaffoldsWithoutAttachments: Any
    keepOnlyFirstFragment: Any
    pruneBeforeFragmenting: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def BRICSScaffoldParams() -> Any: ...
def CreateScaffoldNetwork(boost, RDKit) -> Any: ...
def UpdateScaffoldNetwork(*args, **kwargs) -> Any: ...



from collections.abc import Sequence
from io import IOBase
import sys
from typing import Optional, TextIO
import numpy as np
import numpy.typing as npt
from rdkit import RDConfig as RDConfig
from rdkit.Chem import FragmentCatalog as FragmentCatalog
from rdkit.Dbase.DbConnection import DbConnect as DbConnect
from rdkit.ML import InfoTheory as InfoTheory

def message(msg, dest=sys.stdout) -> None: ...
def BuildCatalog(
    suppl,
    maxPts: int = -1,
    groupFileName: Optional[str] = None,
    minPath: int = 2,
    maxPath: int = 6,
    reportFreq: int = 10,
):
    """
    builds a fragment catalog from a set of molecules in a delimited text block

      **Arguments**

        - suppl: a mol supplier

        - maxPts: (optional) if provided, this will set an upper bound on the
          number of points to be considered

        - groupFileName: (optional) name of the file containing functional group
          information

        - minPath, maxPath: (optional) names of the minimum and maximum path lengths
          to be considered

        - reportFreq: (optional) how often to display status information

      **Returns**

        a FragmentCatalog

    """

def ScoreMolecules(
    suppl,
    catalog,
    maxPts: int = -1,
    actName: str = "",
    acts: Optional[Sequence[int]] = None,
    nActs: int = 2,
    reportFreq: int = 10,
) -> tuple[npt.NDArray[np.int32], Sequence[Sequence[int]]]:
    """
    scores the compounds in a supplier using a catalog

      **Arguments**

        - suppl: a mol supplier

        - catalog: the FragmentCatalog

        - maxPts: (optional) the maximum number of molecules to be
          considered

        - actName: (optional) the name of the molecule's activity property.
          If this is not provided, the molecule's last property will be used.

        - acts: (optional) a sequence of activity values (integers).
          If not provided, the activities will be read from the molecules.

        - nActs: (optional) number of possible activity values

        - reportFreq: (optional) how often to display status information

      **Returns**

        a 2-tuple:

          1) the results table (a 3D array of ints nBits x 2 x nActs)

          2) a list containing the on bit lists for each molecule

    """

def ScoreFromLists(
    bitLists: Sequence[Sequence[int]],
    suppl,
    catalog,
    maxPts: int = -1,
    actName: str = "",
    acts: Optional[Sequence[int]] = None,
    nActs: int = 2,
    reportFreq: int = 10,
) -> tuple[npt.NDArray[np.int32]]:
    """
    similar to _ScoreMolecules()_, but uses pre-calculated bit lists
    for the molecules (this speeds things up a lot)

      **Arguments**

        - bitLists: sequence of on bit sequences for the input molecules

        - suppl: the input supplier (we read activities from here)

        - catalog: the FragmentCatalog

        - maxPts: (optional) the maximum number of molecules to be
          considered

        - actName: (optional) the name of the molecule's activity property.
          If this is not provided, the molecule's last property will be used.

        - nActs: (optional) number of possible activity values

        - reportFreq: (optional) how often to display status information

      **Returns**

         the results table (a 3D array of ints nBits x 2 x nActs)

    """

def CalcGains(
    suppl,
    catalog,
    topN: int = -1,
    actName: str = "",
    acts: Optional[Sequence[int]] = None,
    nActs: int = 2,
    reportFreq: int = 10,
    biasList=None,
    collectFps: int = 0,
):
    """
    calculates info gains by constructing fingerprints
    *DOC*

    Returns a 2-tuple:
        1) gains matrix
        2) list of fingerprints
    """

def CalcGainsFromFps(
    suppl,
    fps,
    topN: int = -1,
    actName: str = "",
    acts: Optional[Sequence[int]] = None,
    nActs: int = 2,
    reportFreq: int = 10,
    biasList=None,
):
    """
    calculates info gains from a set of fingerprints

    *DOC*
    """

def OutputGainsData(outF: IOBase, gains, cat, nActs: int = 2) -> None: ...
def ProcessGainsData(inF: IOBase, delim: str = ",", idCol: int = 0, gainCol: int = 1):
    """reads a list of ids and info gains out of an input file"""

def ShowDetails(
    catalog,
    gains,
    nToDo: int = -1,
    outF: TextIO = sys.stdout,
    idCol: int = 0,
    gainCol: int = 1,
    outDelim: str = ",",
) -> None:
    """
    gains should be a sequence of sequences.  The idCol entry of each
    sub-sequence should be a catalog ID.  _ProcessGainsData()_ provides
    suitable input.
    """

def SupplierFromDetails(details): ...
def Usage() -> None: ...

class RunDetails:
    numMols: int = -1
    doBuild: int = 0
    doSigs: int = 0
    doScore: int = 0
    doGains: int = 0
    doDetails: int = 0
    catalogName: Optional[str] = None
    onBitsName: Optional[str] = None
    scoresName: Optional[str] = None
    gainsName: Optional[str] = None
    dbName: str = ""
    tableName: Optional[str] = None
    detailsName: Optional[str] = None
    inFileName: Optional[str] = None
    fpName: Optional[str] = None
    minPath: int = 2
    maxPath: int = 6
    smiCol: int = 1
    actCol: int = -1
    nameCol: int = -1
    hasTitle: int = 1
    nActs: int = 2
    nBits: int = -1
    delim: str = ","
    biasList: Optional[tuple] = None
    topN: int = -1

def ParseArgs(details: RunDetails) -> None: ...



from typing import Any, ClassVar, Sequence

from typing import overload

from rdkit.Chem.rdchem import Mol

SANITIZE_ADJUST_REACTANTS: SanitizeFlags
SANITIZE_ALL: SanitizeFlags
SANITIZE_ATOM_MAPS: SanitizeFlags
SANITIZE_MERGEHS: SanitizeFlags
SANITIZE_NONE: SanitizeFlags
SANITIZE_RGROUP_NAMES: SanitizeFlags

class CartesianProductStrategy(EnumerationStrategyBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __copy__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class ChemicalReaction:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddAgentTemplate(cls, RDKit, boost) -> Any: ...
    @classmethod
    def AddProductTemplate(cls, RDKit, boost) -> Any: ...
    @classmethod
    def AddReactantTemplate(cls, RDKit, boost) -> Any: ...
    @classmethod
    def AddRecursiveQueriesToReaction(cls, RDKit) -> Any: ...
    @classmethod
    def ClearComputedProps(cls, RDKit) -> Any: ...
    @classmethod
    def ClearProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetAgentTemplate(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetAgents(cls, RDKit) -> Any: ...
    @classmethod
    def GetBoolProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetDoubleProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetIntProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetNumAgentTemplates(cls, RDKit) -> Any: ...
    @classmethod
    def GetNumProductTemplates(cls, RDKit) -> Any: ...
    @classmethod
    def GetNumReactantTemplates(cls, RDKit) -> Any: ...
    @classmethod
    def GetProductTemplate(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetProducts(cls, RDKit) -> Any: ...
    @classmethod
    def GetProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetPropNames(cls, RDKit) -> Any: ...
    @classmethod
    def GetPropsAsDict(cls, RDKit) -> Any: ...
    @classmethod
    def GetReactantTemplate(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetReactants(cls, RDKit) -> Any: ...
    @classmethod
    def GetReactingAtoms(cls, RDKit) -> Any: ...
    @classmethod
    def GetUnsignedProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def HasProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Initialize(cls, RDKit) -> Any: ...
    @classmethod
    def IsInitialized(cls, RDKit) -> Any: ...
    @classmethod
    def IsMoleculeAgent(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def IsMoleculeProduct(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def IsMoleculeReactant(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def RemoveAgentTemplates(cls, RDKit) -> Any: ...
    @classmethod
    def RemoveUnmappedProductTemplates(cls, RDKit) -> Any: ...
    @classmethod
    def RemoveUnmappedReactantTemplates(cls, RDKit) -> Any: ...
    @classmethod
    def RunReactant(cls, RDKit, boost, unsignedint) -> Any: ...
    @classmethod
    def RunReactantInPlace(cls, *args, **kwargs) -> Any: ...
    def RunReactants(
        self, reactants: Sequence[Mol], max_products=1000
    ) -> tuple[tuple[Mol]]: ...
    @classmethod
    def SetBoolProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetDoubleProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetIntProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetUnsignedProp(cls, *args, **kwargs) -> Any: ...
    @overload
    @classmethod
    def ToBinary(cls, RDKit) -> Any: ...
    @overload
    @classmethod
    def ToBinary(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def Validate(cls, RDKit) -> Any: ...
    @classmethod
    def _getImplicitPropertiesFlag(cls, RDKit) -> Any: ...
    @classmethod
    def _setImplicitPropertiesFlag(cls, RDKit, bool) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class EnumerateLibrary(EnumerateLibraryBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetReagents(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class EnumerateLibraryBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetEnumerator(cls, RDKit) -> Any: ...
    @classmethod
    def GetPosition(cls, RDKit) -> Any: ...
    @classmethod
    def GetReaction(cls, RDKit) -> Any: ...
    @classmethod
    def GetState(cls, RDKit) -> Any: ...
    @classmethod
    def InitFromString(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def ResetState(cls, RDKit) -> Any: ...
    @classmethod
    def Serialize(cls, RDKit) -> Any: ...
    @classmethod
    def SetState(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def next(cls, RDKit) -> Any: ...
    @classmethod
    def nextSmiles(cls, RDKit) -> Any: ...
    @classmethod
    def __bool__(cls, RDKit) -> Any: ...
    @classmethod
    def __iter__(cls, boost) -> Any: ...
    @classmethod
    def __next__(cls, RDKit) -> Any: ...
    @classmethod
    def __nonzero__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class EnumerationParams:
    __instance_size__: ClassVar[int] = ...
    reagentMaxMatchCount: Any
    sanePartialProducts: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class EnumerationStrategyBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetNumPermutations(cls, RDKit) -> Any: ...
    @classmethod
    def GetPosition(cls, RDKit) -> Any: ...
    @classmethod
    def Initialize(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Skip(cls, RDKit, unsignedlong) -> Any: ...
    @classmethod
    def Type(cls, RDKit) -> Any: ...
    @classmethod
    def next(cls, RDKit) -> Any: ...
    @classmethod
    def __bool__(cls, RDKit) -> Any: ...
    @classmethod
    def __copy__(cls, RDKit) -> Any: ...
    @classmethod
    def __next__(cls, RDKit) -> Any: ...
    @classmethod
    def __nonzero__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class EvenSamplePairsStrategy(EnumerationStrategyBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def Stats(cls, RDKit) -> Any: ...
    @classmethod
    def __copy__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FingerprintType:
    AtomPairFP: ClassVar[FingerprintType] = ...
    MorganFP: ClassVar[FingerprintType] = ...
    PatternFP: ClassVar[FingerprintType] = ...
    RDKitFP: ClassVar[FingerprintType] = ...
    TopologicalTorsion: ClassVar[FingerprintType] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class MOL_SPTR_VECT:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class RandomSampleAllBBsStrategy(EnumerationStrategyBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __copy__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class RandomSampleStrategy(EnumerationStrategyBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __copy__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class ReactionFingerprintParams:
    __instance_size__: ClassVar[int] = ...
    agentWeight: Any
    bitRatioAgents: Any
    fpSize: Any
    fpType: Any
    includeAgents: Any
    nonAgentWeight: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SanitizeFlags:
    SANITIZE_ADJUST_REACTANTS: ClassVar[SanitizeFlags] = ...
    SANITIZE_ALL: ClassVar[SanitizeFlags] = ...
    SANITIZE_ATOM_MAPS: ClassVar[SanitizeFlags] = ...
    SANITIZE_MERGEHS: ClassVar[SanitizeFlags] = ...
    SANITIZE_NONE: ClassVar[SanitizeFlags] = ...
    SANITIZE_RGROUP_NAMES: ClassVar[SanitizeFlags] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class VectMolVect:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class VectSizeT:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class VectorOfStringVectors:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

def Compute2DCoordsForReaction(RDKit) -> Any: ...
def CreateDifferenceFingerprintForReaction(RDKit) -> Any: ...
def CreateStructuralFingerprintForReaction(RDKit) -> Any: ...
def EnumerateLibraryCanSerialize(*args, **kwargs) -> Any: ...
def GetChemDrawRxnAdjustParams() -> Any: ...
def GetDefaultAdjustParams() -> Any: ...
def HasAgentTemplateSubstructMatch(*args, **kwargs) -> Any: ...
def HasProductTemplateSubstructMatch(*args, **kwargs) -> Any: ...
def HasReactantTemplateSubstructMatch(*args, **kwargs) -> Any: ...
def HasReactionAtomMapping(RDKit) -> Any: ...
def HasReactionSubstructMatch(*args, **kwargs) -> Any: ...
def IsReactionTemplateMoleculeAgent(RDKit, double) -> Any: ...
def MatchOnlyAtRgroupsAdjustParams() -> Any: ...
def PreprocessReaction(RDKit) -> Any: ...
def ReactionFromMolecule(RDKit) -> Any: ...
def ReactionFromPNGFile(*args, **kwargs) -> Any: ...
def ReactionFromPNGString(*args, **kwargs) -> Any: ...
def ReactionFromRxnBlock(*args, **kwargs) -> Any: ...
def ReactionFromRxnFile(*args, **kwargs) -> Any: ...
def ReactionFromSmarts(*args, **kwargs) -> Any: ...
def ReactionMetadataToPNGFile(RDKit, boost) -> Any: ...
def ReactionMetadataToPNGString(RDKit, boost) -> Any: ...
def ReactionToMolecule(RDKit) -> Any: ...
def ReactionToRxnBlock(RDKit) -> Any: ...
def ReactionToSmarts(RDKit) -> Any: ...
def ReactionToSmiles(RDKit) -> Any: ...
def ReactionToV3KRxnBlock(RDKit) -> Any: ...
def ReduceProductToSideChains(boost) -> Any: ...
def RemoveMappingNumbersFromReactions(RDKit) -> Any: ...
def SanitizeRxn(*args, **kwargs) -> Any: ...
def UpdateProductsStereochemistry(RDKit) -> Any: ...



from _typeshed import Incomplete

class Color:
    def __init__(self, red: int = ..., green: int = ..., blue: int = ...) -> None: ...
    def __setattr__(self, name, value) -> None: ...
    def __mul__(self, x): ...
    def __rmul__(self, x): ...
    def __truediv__(self, x): ...
    def __div__(self, x): ...
    def __rdiv__(self, x): ...
    def __add__(self, x): ...
    def __sub__(self, x): ...
    def __repr__(self): ...
    def __hash__(self): ...
    def __cmp__(self, other): ...
    def toHexRGB(self): ...
    def toHexStr(self): ...

def HexColor(val): ...

aliceblue: Incomplete
antiquewhite: Incomplete
aqua: Incomplete
aquamarine: Incomplete
azure: Incomplete
beige: Incomplete
bisque: Incomplete
black: Incomplete
blanchedalmond: Incomplete
blue: Incomplete
blueviolet: Incomplete
brown: Incomplete
burlywood: Incomplete
cadetblue: Incomplete
chartreuse: Incomplete
chocolate: Incomplete
coral: Incomplete
cornflower: Incomplete
cornsilk: Incomplete
crimson: Incomplete
cyan: Incomplete
darkblue: Incomplete
darkcyan: Incomplete
darkgoldenrod: Incomplete
darkgray: Incomplete
darkgreen: Incomplete
darkkhaki: Incomplete
darkmagenta: Incomplete
darkolivegreen: Incomplete
darkorange: Incomplete
darkorchid: Incomplete
darkred: Incomplete
darksalmon: Incomplete
darkseagreen: Incomplete
darkslateblue: Incomplete
darkslategray: Incomplete
darkturquoise: Incomplete
darkviolet: Incomplete
deeppink: Incomplete
deepskyblue: Incomplete
dimgray: Incomplete
dodgerblue: Incomplete
firebrick: Incomplete
floralwhite: Incomplete
forestgreen: Incomplete
fuchsia: Incomplete
gainsboro: Incomplete
ghostwhite: Incomplete
gold: Incomplete
goldenrod: Incomplete
gray: Incomplete
grey = gray
green: Incomplete
greenyellow: Incomplete
honeydew: Incomplete
hotpink: Incomplete
indianred: Incomplete
indigo: Incomplete
ivory: Incomplete
khaki: Incomplete
lavender: Incomplete
lavenderblush: Incomplete
lawngreen: Incomplete
lemonchiffon: Incomplete
lightblue: Incomplete
lightcoral: Incomplete
lightcyan: Incomplete
lightgoldenrodyellow: Incomplete
lightgreen: Incomplete
lightgrey: Incomplete
lightpink: Incomplete
lightsalmon: Incomplete
lightseagreen: Incomplete
lightskyblue: Incomplete
lightslategray: Incomplete
lightsteelblue: Incomplete
lightyellow: Incomplete
lime: Incomplete
limegreen: Incomplete
linen: Incomplete
magenta: Incomplete
maroon: Incomplete
mediumaquamarine: Incomplete
mediumblue: Incomplete
mediumorchid: Incomplete
mediumpurple: Incomplete
mediumseagreen: Incomplete
mediumslateblue: Incomplete
mediumspringgreen: Incomplete
mediumturquoise: Incomplete
mediumvioletred: Incomplete
midnightblue: Incomplete
mintcream: Incomplete
mistyrose: Incomplete
moccasin: Incomplete
navajowhite: Incomplete
navy: Incomplete
oldlace: Incomplete
olive: Incomplete
olivedrab: Incomplete
orange: Incomplete
orangered: Incomplete
orchid: Incomplete
palegoldenrod: Incomplete
palegreen: Incomplete
paleturquoise: Incomplete
palevioletred: Incomplete
papayawhip: Incomplete
peachpuff: Incomplete
peru: Incomplete
pink: Incomplete
plum: Incomplete
powderblue: Incomplete
purple: Incomplete
red: Incomplete
rosybrown: Incomplete
royalblue: Incomplete
saddlebrown: Incomplete
salmon: Incomplete
sandybrown: Incomplete
seagreen: Incomplete
seashell: Incomplete
sienna: Incomplete
silver: Incomplete
skyblue: Incomplete
slateblue: Incomplete
slategray: Incomplete
snow: Incomplete
springgreen: Incomplete
steelblue: Incomplete
tan: Incomplete
teal: Incomplete
thistle: Incomplete
tomato: Incomplete
turquoise: Incomplete
violet: Incomplete
wheat: Incomplete
white: Incomplete
whitesmoke: Incomplete
yellow: Incomplete
yellowgreen: Incomplete
transparent: Incomplete



from math import *
from _typeshed import Incomplete
from rdkit.sping import pid as pid

def _ColorToQt(color): ...

class QCanvasRotText:
    _angle: Incomplete
    def __init__(self, txt, canvas, angle: int = ...) -> None: ...
    def draw(self, qP) -> None: ...

class QtCanvas(pid.Canvas):
    size: Incomplete
    _scene: Incomplete
    _brush: Incomplete
    _pen: Incomplete
    _font: Incomplete
    objs: Incomplete
    nObjs: int
    def __init__(
        self, scene, size: Incomplete | None = ..., name: str = ...
    ) -> None: ...
    def _initOutput(self) -> None: ...
    def _adjustFont(self, font) -> None: ...
    def clear(self) -> None: ...
    def flush(self) -> None: ...
    def save(
        self, file: Incomplete | None = ..., format: Incomplete | None = ...
    ) -> None: ...
    def drawLine(
        self,
        x1,
        y1,
        x2,
        y2,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawPolygon(
        self,
        pointlist,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor=...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def stringBox(self, s, font: Incomplete | None = ...): ...
    def stringWidth(self, s, font: Incomplete | None = ...): ...
    def fontAscent(self, font: Incomplete | None = ...): ...
    def fontDescent(self, font: Incomplete | None = ...): ...

def test(canvas): ...
def dashtest(canvas): ...



from _typeshed import Incomplete

class ActFunc:
    def __call__(self, x): ...

class Sigmoid(ActFunc):
    def Eval(self, x): ...
    def Deriv(self, x): ...
    def DerivFromVal(self, val): ...
    beta: Incomplete
    def __init__(self, beta: float = ...) -> None: ...

class TanH(ActFunc):
    def Eval(self, x): ...
    def Deriv(self, x): ...
    def DerivFromVal(self, val): ...
    beta: Incomplete
    def __init__(self, beta: float = ...) -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.VLib.Supply import SupplyNode as SupplyNode

class SmilesSupplyNode(SupplyNode):
    _fileName: Incomplete
    _supplier: Incomplete
    def __init__(
        self,
        fileName,
        delim: str = ...,
        nameColumn: int = ...,
        smilesColumn: int = ...,
        titleLine: int = ...,
        **kwargs
    ) -> None: ...
    def reset(self) -> None: ...
    def next(self): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from typing import Any, ClassVar, overload

class MolChemicalFeature:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def ClearCache(cls, RDKit) -> Any: ...
    @classmethod
    def GetActiveConformer(cls, RDKit) -> Any: ...
    @classmethod
    def GetAtomIds(cls, RDKit) -> Any: ...
    @classmethod
    def GetFactory(cls, RDKit) -> Any: ...
    @classmethod
    def GetFamily(cls, RDKit) -> Any: ...
    @classmethod
    def GetId(cls, RDKit) -> Any: ...
    @classmethod
    def GetMol(cls, RDKit) -> Any: ...
    @overload
    @classmethod
    def GetPos(cls, RDKit, int) -> Any: ...
    @overload
    @classmethod
    def GetPos(cls, RDKit) -> Any: ...
    @classmethod
    def GetType(cls, RDKit) -> Any: ...
    @classmethod
    def SetActiveConformer(cls, RDKit, int) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolChemicalFeatureFactory:
    GetFeaturesForMol: ClassVar[function] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetFeatureDefs(cls, RDKit) -> Any: ...
    @classmethod
    def GetFeatureFamilies(cls, RDKit) -> Any: ...
    @classmethod
    def GetMolFeature(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetNumFeatureDefs(cls, RDKit) -> Any: ...
    @classmethod
    def GetNumMolFeatures(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def BuildFeatureFactory(*args, **kwargs) -> Any: ...
def BuildFeatureFactoryFromString(arg1: str) -> MolChemicalFeatureFactory: ...
def GetAtomMatch(boost) -> Any: ...



from _typeshed import Incomplete
from rdkit.VLib.Node import VLibNode as VLibNode

class SupplyNode(VLibNode):
    _contents: Incomplete
    _pos: int
    def __init__(self, contents: Incomplete | None = ..., **kwargs) -> None: ...
    def reset(self) -> None: ...
    def next(self): ...
    def AddParent(self, parent, notify: int = ...) -> None: ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.Chem.Pharm2D import SigFactory as SigFactory, Utils as Utils
from rdkit.RDLogger import logger as logger

_verbose: int

def _ShortestPathsMatch(match, featureSet, sig, dMat, sigFactory): ...
def Gen2DFingerprint(
    mol,
    sigFactory,
    perms: Incomplete | None = ...,
    dMat: Incomplete | None = ...,
    bitInfo: Incomplete | None = ...,
): ...



from _typeshed import Incomplete
from rdkit import (
    Chem as Chem,
    Geometry as Geometry,
    RDConfig as RDConfig,
    rdBase as rdBase,
)
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors, rdchem as rdchem

def _doMatch(inv, atoms): ...
def _doNotMatch(inv, atoms): ...
def _doMatchExcept1(inv, atoms): ...
def _getAtomInvariantsWithRadius(mol, radius): ...
def _getHeavyAtomNeighbors(atom1, aid2: int = ...): ...
def _getIndexforTorsion(neighbors, inv): ...
def _getBondsForTorsions(mol, ignoreColinearBonds): ...
def CalculateTorsionLists(
    mol, maxDev: str = ..., symmRadius: int = ..., ignoreColinearBonds: bool = ...
): ...
def _getTorsionAtomPositions(atoms, conf): ...
def CalculateTorsionAngles(mol, tors_list, tors_list_rings, confId: int = ...): ...
def _findCentralBond(mol, distmat): ...
def _calculateBeta(mol, distmat, aid1): ...
def CalculateTorsionWeights(
    mol, aid1: int = ..., aid2: int = ..., ignoreColinearBonds: bool = ...
): ...
def CalculateTFD(torsions1, torsions2, weights: Incomplete | None = ...): ...
def _getSameAtomOrder(mol1, mol2): ...
def GetTFDBetweenConformers(
    mol,
    confIds1,
    confIds2,
    useWeights: bool = ...,
    maxDev: str = ...,
    symmRadius: int = ...,
    ignoreColinearBonds: bool = ...,
): ...
def GetTFDBetweenMolecules(
    mol1,
    mol2,
    confId1: int = ...,
    confId2: int = ...,
    useWeights: bool = ...,
    maxDev: str = ...,
    symmRadius: int = ...,
    ignoreColinearBonds: bool = ...,
): ...
def GetTFDMatrix(
    mol,
    useWeights: bool = ...,
    maxDev: str = ...,
    symmRadius: int = ...,
    ignoreColinearBonds: bool = ...,
): ...



from rdkit.ML.InfoTheory import entropy as entropy

def FormCounts(
    bitVects, actVals, whichBit, nPossibleActs, nPossibleBitVals: int = ...
): ...
def CalcInfoGains(bitVects, actVals, nPossibleActs, nPossibleBitVals: int = ...): ...
def RankBits(bitVects, actVals, nPossibleBitVals: int = ..., metricFunc=...): ...
def AnalyzeSparseVects(bitVects, actVals): ...
def SparseRankBits(bitVects, actVals, metricFunc=...): ...



from _typeshed import Incomplete

def cmp(t1, t2): ...

CMPTOL: float

class Cluster:
    metric: Incomplete
    children: Incomplete
    pos: Incomplete
    index: Incomplete
    name: Incomplete
    _points: Incomplete
    _pointsPositions: Incomplete
    data: Incomplete
    def __init__(
        self,
        metric: float = ...,
        children: Incomplete | None = ...,
        position: Incomplete | None = ...,
        index: int = ...,
        name: Incomplete | None = ...,
        data: Incomplete | None = ...,
    ) -> None: ...
    def SetMetric(self, metric) -> None: ...
    def GetMetric(self): ...
    def SetIndex(self, index) -> None: ...
    def GetIndex(self): ...
    def SetPosition(self, pos) -> None: ...
    def GetPosition(self): ...
    def GetPointsPositions(self): ...
    def GetPoints(self): ...
    def FindSubtree(self, index): ...
    def _GenPoints(self): ...
    def AddChild(self, child) -> None: ...
    def AddChildren(self, children) -> None: ...
    def RemoveChild(self, child) -> None: ...
    def GetChildren(self): ...
    def SetData(self, data) -> None: ...
    def GetData(self): ...
    def SetName(self, name) -> None: ...
    def GetName(self): ...
    def Print(
        self, level: int = ..., showData: int = ..., offset: str = ...
    ) -> None: ...
    def Compare(self, other, ignoreExtras: int = ...): ...
    _len: Incomplete
    def _UpdateLength(self) -> None: ...
    def IsTerminal(self): ...
    def __len__(self) -> int: ...
    def __cmp__(self, other): ...



from math import *
from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig

__DEBUG: bool
knownMethods: Incomplete

def HAS(strArg, composList, atomDict): ...
def SUM(strArg, composList, atomDict): ...
def MEAN(strArg, composList, atomDict): ...

AVG = MEAN

def DEV(strArg, composList, atomDict): ...
def MIN(strArg, composList, atomDict): ...
def MAX(strArg, composList, atomDict): ...
def _SubForAtomicVars(cExpr, varList, dictName): ...
def _SubForCompoundDescriptors(cExpr, varList, dictName): ...
def _SubMethodArgs(cExpr, knownMethods): ...
def CalcSingleCompoundDescriptor(compos, argVect, atomDict, propDict): ...
def CalcMultipleCompoundsDescriptor(composVect, argVect, atomDict, propDictList): ...
def _exampleCode() -> None: ...



from rdkit.Chem.rdChemicalFeatures import *
from rdkit.Chem.rdMolChemicalFeatures import *
from rdkit.Chem import Mol

def MCFF_GetFeaturesForMol(
    self, mol: Mol, includeOnly: str = "", confId: int = -1
) -> tuple[MolChemicalFeature, ...]: ...



from rdkit.sping.pid import *
from . import psmetrics as psmetrics
from _typeshed import Incomplete

class PostScriptLevelException(ValueError): ...

linesep: str
PiddleLegalFonts: Incomplete
Roman: str
Bold: str
Italic: str
PSFontMapStdEnc: Incomplete
PSFontMapLatin1Enc: Incomplete

def latin1FontEncoding(fontname): ...
def dashLineDefinition(): ...

class PsDSC:
    def __init__(self) -> None: ...
    def documentHeader(self): ...
    def boundingBoxStr(self, x0, y0, x1, y1): ...
    inPageFlag: int
    def BeginPageStr(self, pageSetupStr, pageName: Incomplete | None = ...): ...
    def EndPageStr(self): ...

class EpsDSC(PsDSC):
    def __init__(self) -> None: ...
    def documentHeader(self): ...

class PSCanvas(Canvas):
    filename: Incomplete
    drawImage: Incomplete
    code: Incomplete
    dsc: Incomplete
    defaultFont: Incomplete
    fontMapEncoding: Incomplete
    _currentFont: Incomplete
    _inDocumentFlag: int
    _inPageFlag: int
    pageNum: int
    def __init__(
        self, size=..., name: str = ..., PostScriptLevel: int = ..., fontMapEncoding=...
    ) -> None: ...
    def psBeginDocument(self) -> None: ...
    def psEndDocument(self) -> None: ...
    def psBeginPage(self, pageName: Incomplete | None = ...) -> None: ...
    def _psPageSetupStr(
        self, pageheight, initialColor, font_family, font_size, line_width
    ): ...
    def psEndPage(self) -> None: ...
    def _findFont(self, font): ...
    def _findExternalFontName(self, font): ...
    def _psNextPage(self) -> None: ...
    def nextPage(self) -> None: ...
    def clear(self) -> None: ...
    _currentColor: Incomplete
    _currentWidth: Incomplete
    def resetToDefaults(self) -> None: ...
    def flush(self) -> None: ...
    def save(
        self, file: Incomplete | None = ..., format: Incomplete | None = ...
    ) -> None: ...
    def stringWidth(self, s, font: Incomplete | None = ...): ...
    def fontAscent(self, font: Incomplete | None = ...): ...
    def fontDescent(self, font: Incomplete | None = ...): ...
    def _updateLineColor(self, color) -> None: ...
    def _updateFillColor(self, color) -> None: ...
    def _updateLineWidth(self, width) -> None: ...
    def _updateFont(self, font) -> None: ...
    def drawLine(
        self,
        x1,
        y1,
        x2,
        y2,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawLines(
        self,
        lineList,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def _escape(self, s): ...
    def _drawStringOneLineNoRot(
        self, s, x, y, font: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def _drawStringOneLine(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def drawString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def drawCurve(
        self,
        x1,
        y1,
        x2,
        y2,
        x3,
        y3,
        x4,
        y4,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawRoundRect(
        self,
        x1,
        y1,
        x2,
        y2,
        rx: int = ...,
        ry: int = ...,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawEllipse(
        self,
        x1,
        y1,
        x2,
        y2,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawArc(
        self,
        x1,
        y1,
        x2,
        y2,
        startAng: int = ...,
        extent: int = ...,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def _genArcCode(self, x1, y1, x2, y2, startAng, extent): ...
    def drawPolygon(
        self,
        pointlist,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawFigure(
        self,
        partList,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def _drawImageLevel1(
        self,
        image,
        x1,
        y1,
        x2: Incomplete | None = ...,
        y2: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def _AsciiHexEncode(self, input): ...
    def _drawImageLevel2(
        self, image, x1, y1, x2: Incomplete | None = ..., y2: Incomplete | None = ...
    ) -> None: ...



from _typeshed import Incomplete
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors
from rdkit.Chem.AtomPairs import Utils as Utils
from rdkit.Chem.rdMolDescriptors import (
    GetHashedTopologicalTorsionFingerprint as GetHashedTopologicalTorsionFingerprint,
    GetTopologicalTorsionFingerprint as GetTopologicalTorsionFingerprint,
)

GetTopologicalTorsionFingerprintAsIntVect: Incomplete

def pyScorePath(mol, path, size, atomCodes: Incomplete | None = ...): ...
def ExplainPathScore(score, size: int = ...): ...
def GetTopologicalTorsionFingerprintAsIds(mol, targetSize: int = ...): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



import io

from rdkit import Chem
from rdkit.Chem import Mol, SDWriter, rdDistGeom


def sample_mol() -> Mol:
    ret = Chem.MolFromSmiles("O=C(O)CCCCO/N=C(\\c1ccnnc1)c1cccc(C(F)(F)F)c1")
    assert ret is not None
    return ret


def frob(mol: Mol):
    for atom in mol.GetAtoms():
        a: Chem.Atom = atom
        _ = a
        n: int = atom.GetAtomicNum()
        assert type(n) == int


def sdwriter():
    t: str = SDWriter.GetText(sample_mol())
    assert type(t) == str

    bs = io.StringIO()
    writer = SDWriter(bs)
    for mol in [sample_mol(), sample_mol(), sample_mol()]:
        writer.write(mol)
    writer.close()
    first = bs.getvalue()

    bs = io.StringIO()
    with SDWriter(bs) as writer:
        for mol in [sample_mol(), sample_mol(), sample_mol()]:
            writer.write(mol)
    second = bs.getvalue()
    assert second == first

    writer = SDWriter(io.StringIO())
    writer.SetProps(["asdf"])
    writer.SetProps(("asdf",))


def conformers():
    mol = sample_mol()
    rdDistGeom.EmbedMolecule(mol)
    conf = mol.GetConformer()
    poses = conf.GetPositions()
    assert len(poses.shape) == 2
    assert poses.shape[1] == 3


def add_hs():
    mol = sample_mol()
    Chem.AddHs(mol)


def embed():
    mol = sample_mol()
    rdDistGeom.EmbedMolecule(mol)


def test_frob():
    frob(sample_mol())
    sdwriter()
    conformers()
    add_hs()
    embed()



from _typeshed import Incomplete
from rdkit import Chem as Chem, DataStructs as DataStructs

similarityMethods: Incomplete
supportedSimilarityMethods: Incomplete

class LayeredOptions:
    loadLayerFlags: int
    searchLayerFlags: int
    minPath: int
    maxPath: int
    fpSize: int
    wordSize: int
    nWords: Incomplete
    @staticmethod
    def GetFingerprint(mol, query: bool = ...): ...
    @staticmethod
    def GetWords(mol, query: bool = ...): ...
    @staticmethod
    def GetQueryText(mol, query: bool = ...): ...

def BuildSigFactory(
    options: Incomplete | None = ...,
    fdefFile: Incomplete | None = ...,
    bins=...,
    skipFeats=...,
): ...
def BuildAtomPairFP(mol): ...
def BuildTorsionsFP(mol): ...
def BuildRDKitFP(mol): ...
def BuildPharm2DFP(mol): ...
def BuildMorganFP(mol): ...
def BuildAvalonFP(mol, smiles: Incomplete | None = ...): ...
def DepickleFP(pkl, similarityMethod): ...



from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors
from rdkit.Chem.AtomPairs import Utils as Utils
from rdkit.Chem.rdMolDescriptors import (
    GetAtomPairFingerprint as GetAtomPairFingerprint,
    GetHashedAtomPairFingerprint as GetHashedAtomPairFingerprint,
)

GetAtomPairFingerprintAsIntVect: Incomplete
numPathBits: Incomplete
_maxPathLen: Incomplete
numFpBits: Incomplete
fpLen: Incomplete

def pyScorePair(
    at1, at2, dist, atomCodes: Incomplete | None = ..., includeChirality: bool = ...
): ...
def ExplainPairScore(score, includeChirality: bool = ...): ...
def GetAtomPairFingerprintAsBitVect(mol): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.DataStructs.BitEnsemble import BitEnsemble as BitEnsemble

def _InitScoreTable(
    self, dbConn, tableName, idInfo: str = ..., actInfo: str = ...
) -> None: ...
def _ScoreToDb(
    self,
    sig,
    dbConn,
    tableName: Incomplete | None = ...,
    id: Incomplete | None = ...,
    act: Incomplete | None = ...,
) -> None: ...



from _typeshed import Incomplete
from rdkit.sping.pid import cm as cm, inch as inch

_W: Incomplete
_H: Incomplete
A6: Incomplete
A5: Incomplete
A4: Incomplete
A3: Incomplete
A2: Incomplete
A1: Incomplete
A0: Incomplete
letter: Incomplete
legal: Incomplete
elevenSeventeen: Incomplete
_BW: Incomplete
_BH: Incomplete
B6: Incomplete
B5: Incomplete
B4: Incomplete
B3: Incomplete
B2: Incomplete
B1: Incomplete
B0: Incomplete



from _typeshed import Incomplete
from rdkit import Chem as Chem, Geometry as Geometry
from rdkit.Chem import rdDepictor as rdDepictor

def AlignDepict(
    mol, core, corePattern: Incomplete | None = ..., acceptFailure: bool = ...
) -> None: ...
def initParser(): ...
def processArgs(args) -> None: ...
def main() -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.VLib.Supply import SupplyNode as SupplyNode

class SDSupplyNode(SupplyNode):
    _fileName: Incomplete
    _supplier: Incomplete
    def __init__(self, fileName, **kwargs) -> None: ...
    def reset(self) -> None: ...
    def next(self): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.Chem import ChemicalFeatures as ChemicalFeatures
from rdkit.Chem.Pharm2D.SigFactory import SigFactory as SigFactory

fdef: str
defaultBins: Incomplete

def _init() -> None: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem

def Convert(
    suppl,
    outFile,
    keyCol: Incomplete | None = ...,
    stopAfter: int = ...,
    includeChirality: bool = ...,
    smilesFrom: str = ...,
) -> None: ...
def initParser(): ...
def existingFile(filename): ...
def main() -> None: ...



from typing import Any

def Compute2DCoords(RDKit) -> Any: ...
def Compute2DCoordsMimicDistmat(RDKit, boost) -> Any: ...
def GenerateDepictionMatching2DStructure(*args, **kwargs) -> Any: ...
def GenerateDepictionMatching3DStructure(*args, **kwargs) -> Any: ...
def NormalizeDepiction(RDKit) -> Any: ...
def SetPreferCoordGen(bool) -> Any: ...
def StraightenDepiction(RDKit) -> Any: ...



from _typeshed import Incomplete
from rdkit.DataStructs import BitUtils as BitUtils
from rdkit.ML.Data import MLData as MLData
from rdkit.utils import fileutils as fileutils

def permutation(nToDo): ...
def WriteData(outFile, varNames, qBounds, examples) -> None: ...
def ReadVars(inFile): ...
def ReadQuantExamples(inFile): ...
def ReadGeneralExamples(inFile): ...
def BuildQuantDataSet(fileName): ...
def BuildDataSet(fileName): ...
def CalcNPossibleUsingMap(
    data, order, qBounds, nQBounds: Incomplete | None = ..., silent: bool = ...
): ...
def WritePickledData(outName, data) -> None: ...
def TakeEnsemble(vect, ensembleIds, isDataVect: bool = ...): ...
def DBToData(
    dbName,
    tableName,
    user: str = ...,
    password: str = ...,
    dupCol: int = ...,
    what: str = ...,
    where: str = ...,
    join: str = ...,
    pickleCol: int = ...,
    pickleClass: Incomplete | None = ...,
    ensembleIds: Incomplete | None = ...,
): ...
def TextToData(reader, ignoreCols=..., onlyCols: Incomplete | None = ...): ...
def TextFileToData(fName, onlyCols: Incomplete | None = ...): ...
def InitRandomNumbers(seed) -> None: ...
def FilterData(
    inData,
    val,
    frac,
    col: int = ...,
    indicesToUse: Incomplete | None = ...,
    indicesOnly: int = ...,
): ...
def CountResults(inData, col: int = ..., bounds: Incomplete | None = ...): ...
def RandomizeActivities(
    dataSet, shuffle: int = ..., runDetails: Incomplete | None = ...
) -> None: ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from typing import Any

def InchiToInchiKey(*args, **kwargs) -> Any: ...
def InchiToMol(*args, **kwargs) -> Any: ...
def MolBlockToInchi(*args, **kwargs) -> Any: ...
def MolToInchi(RDKit) -> Any: ...
def MolToInchiKey(RDKit) -> Any: ...



from _typeshed import Incomplete
from collections.abc import Generator
from rdkit import Chem as Chem, Geometry as Geometry, RDLogger as RDLogger
from rdkit.Chem.Subshape import SubshapeObjects as SubshapeObjects

logger: Incomplete

class SubshapeAlignment:
    transform: Incomplete
    triangleSSD: Incomplete
    targetTri: Incomplete
    queryTri: Incomplete
    alignedConfId: int
    dirMatch: float
    shapeDist: float

def _getAllTriangles(
    pts, orderedTraversal: bool = ...
) -> Generator[Incomplete, None, None]: ...

class SubshapeDistanceMetric:
    TANIMOTO: int
    PROTRUDE: int

def GetShapeShapeDistance(s1, s2, distMetric): ...
def ClusterAlignments(
    mol,
    alignments,
    builder,
    neighborTol: float = ...,
    distMetric=...,
    tempConfId: int = ...,
): ...
def TransformMol(mol, tform, confId: int = ..., newConfId: int = ...) -> None: ...

class SubshapeAligner:
    triangleRMSTol: float
    distMetric: Incomplete
    shapeDistTol: float
    numFeatThresh: int
    dirThresh: float
    edgeTol: float
    coarseGridToleranceMult: float
    medGridToleranceMult: float
    def GetTriangleMatches(
        self, target, query
    ) -> Generator[Incomplete, None, None]: ...
    def _checkMatchFeatures(self, targetPts, queryPts, alignment): ...
    def PruneMatchesUsingFeatures(
        self, target, query, alignments, pruneStats: Incomplete | None = ...
    ) -> None: ...
    def _checkMatchDirections(self, targetPts, queryPts, alignment): ...
    def PruneMatchesUsingDirection(
        self, target, query, alignments, pruneStats: Incomplete | None = ...
    ) -> None: ...
    def _addCoarseAndMediumGrids(self, mol, tgt, confId, builder) -> None: ...
    def _checkMatchShape(
        self,
        targetMol,
        target,
        queryMol,
        query,
        alignment,
        builder,
        targetConf,
        queryConf,
        pruneStats: Incomplete | None = ...,
        tConfId: int = ...,
    ): ...
    def PruneMatchesUsingShape(
        self,
        targetMol,
        target,
        queryMol,
        query,
        builder,
        alignments,
        tgtConf: int = ...,
        queryConf: int = ...,
        pruneStats: Incomplete | None = ...,
    ) -> None: ...
    def GetSubshapeAlignments(
        self,
        targetMol,
        target,
        queryMol,
        query,
        builder,
        tgtConf: int = ...,
        queryConf: int = ...,
        pruneStats: Incomplete | None = ...,
    ): ...
    def __call__(
        self,
        targetMol,
        target,
        queryMol,
        query,
        builder,
        tgtConf: int = ...,
        queryConf: int = ...,
        pruneStats: Incomplete | None = ...,
    ) -> Generator[Incomplete, None, None]: ...



from rdkit.ML.DecTree import CrossValidate as CrossValidate, DecTree as DecTree

_verbose: int

def MaxCount(examples): ...
def _GetLocalError(node): ...
def _Pruner(node, level: int = ...): ...
def PruneTree(tree, trainExamples, testExamples, minimizeTestErrorOnly: int = ...): ...
def _testRandom() -> None: ...
def _testSpecific() -> None: ...
def _testChain() -> None: ...



from _typeshed import Incomplete
from rdkit import RDConfig as RDConfig

_atomDbName: Incomplete

def GetAtomicData(
    atomDict,
    descriptorsDesired,
    dBase=...,
    table: str = ...,
    where: str = ...,
    user: str = ...,
    password: str = ...,
    includeElCounts: int = ...,
) -> None: ...
def SplitComposition(compStr): ...
def ConfigToNumElectrons(config, ignoreFullD: int = ..., ignoreFullF: int = ...): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.VLib.Filter import FilterNode as FilterNode

class SmartsFilter(FilterNode):
    def __init__(self, patterns=..., counts=..., **kwargs) -> None: ...
    _patterns: Incomplete
    def _initPatterns(self, patterns, counts) -> None: ...
    def filter(self, cmpd): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from . import InteractiveRenderer as InteractiveRenderer
from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import (
    Draw as Draw,
    rdChemReactions as rdChemReactions,
    rdchem as rdchem,
)
from rdkit.Chem.Draw import rdMolDraw2D as rdMolDraw2D

_canUse3D: bool
molSize: Incomplete
highlightSubstructs: bool
kekulizeStructures: bool
highlightByReactant: bool
ipython_useSVG: bool
ipython_showProperties: bool
ipython_maxProperties: int
ipython_3d: bool
molSize_3d: Incomplete
drawing_type_3d: str
bgcolor_3d: str
drawOptions: Incomplete

def addMolToView(
    mol, view, confId: int = ..., drawAs: Incomplete | None = ...
) -> None: ...
def drawMol3D(
    m,
    view: Incomplete | None = ...,
    confId: int = ...,
    drawAs: Incomplete | None = ...,
    bgColor: Incomplete | None = ...,
    size: Incomplete | None = ...,
): ...
def _toJSON(mol): ...
def _wrapHTMLIntoTable(html): ...
def _toHTML(mol): ...
def _toPNG(mol): ...
def _toSVG(mol): ...
def _toReactionPNG(rxn): ...
def _toReactionSVG(rxn): ...
def _toMolBundlePNG(bundle): ...
def _toMolBundleSVG(bundle): ...
def _GetSubstructMatch(mol, query, *args, **kwargs): ...
def _GetSubstructMatches(mol, query, *args, **kwargs): ...
def display_pil_image(img): ...

_MolsToGridImageSaved: Incomplete

def ShowMols(mols, maxMols: int = ..., **kwargs): ...
def _DrawBit(fn, *args, **kwargs): ...
def _DrawBits(fn, *args, **kwargs): ...

_DrawMorganBitSaved: Incomplete

def DrawMorganBit(mol, bitId, bitInfo, drawOptions=..., **kwargs): ...

_DrawMorganBitsSaved: Incomplete

def DrawMorganBits(*args, drawOptions=..., **kwargs): ...

_DrawRDKitBitSaved: Incomplete

def DrawRDKitBit(mol, bitId, bitInfo, drawOptions=..., **kwargs): ...

_DrawRDKitBitsSaved: Incomplete

def DrawRDKitBits(*args, drawOptions=..., **kwargs): ...

_rendererInstalled: bool

def EnableSubstructMatchRendering() -> None: ...

_methodsToDelete: Incomplete

def InstallIPythonRenderer(): ...
def DisableSubstructMatchRendering() -> None: ...
def UninstallIPythonRenderer() -> None: ...



from typing import Any, ClassVar

class CoordGenParams:
    __instance_size__: ClassVar[int] = ...
    coordgenScaling: Any
    dbg_useConstrained: Any
    dbg_useFixed: Any
    minimizerPrecision: Any
    templateFileDir: Any
    treatNonterminalBondsToMetalAsZOBs: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def SetCoordMap(cls, RDKit, boost) -> Any: ...
    @classmethod
    def SetTemplateMol(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def sketcherBestPrecision(self) -> Any: ...
    @property
    def sketcherCoarsePrecision(self) -> Any: ...
    @property
    def sketcherQuickPrecision(self) -> Any: ...
    @property
    def sketcherStandardPrecision(self) -> Any: ...

def AddCoords(RDKit) -> Any: ...
def SetDefaultTemplateFileDir(*args, **kwargs) -> Any: ...



from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs
from rdkit.VLib.Node import VLibNode as VLibNode

class DbFpSupplier(VLibNode):
    _usePickles: Incomplete
    _data: Incomplete
    _fpColName: Incomplete
    _colNames: Incomplete
    fpCol: Incomplete
    _numProcessed: int
    def __init__(
        self, dbResults, fpColName: str = ..., usePickles: bool = ...
    ) -> None: ...
    def GetColumnNames(self): ...
    def _BuildFp(self, data): ...
    def next(self): ...
    __next__: Incomplete

class ForwardDbFpSupplier(DbFpSupplier):
    def __init__(self, *args, **kwargs) -> None: ...
    _dataIter: Incomplete
    def reset(self) -> None: ...
    def NextItem(self): ...

class RandomAccessDbFpSupplier(DbFpSupplier):
    def __init__(self, *args, **kwargs) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, idx): ...
    _pos: int
    def reset(self) -> None: ...
    def NextItem(self): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit.Chem.Pharm2D import Matcher as Matcher, SigFactory as SigFactory

class Generator:
    sigFactory: Incomplete
    mol: Incomplete
    dMat: Incomplete
    bits: Incomplete
    pattMatches: Incomplete
    def __init__(
        self, sigFactory, mol, dMat: Incomplete | None = ..., bitCache: bool = ...
    ) -> None: ...
    def GetBit(self, idx): ...
    def __len__(self) -> int: ...
    def __getitem__(self, itm): ...



from typing import Any

class TautomerQuery:
    def __init__(self, RDKit): ...
    @classmethod
    def GetModifiedAtoms(cls, RDKit) -> Any: ...
    @classmethod
    def GetModifiedBonds(cls, RDKit) -> Any: ...
    @classmethod
    def GetSubstructMatch(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetSubstructMatches(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetSubstructMatchesWithTautomers(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetTautomers(cls, RDKit) -> Any: ...
    @classmethod
    def GetTemplateMolecule(cls, RDKit) -> Any: ...
    @classmethod
    def IsSubstructOf(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def PatternFingerprintTemplate(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def PatternFingerprintTautomerTarget(RDKit) -> Any: ...



from typing import Optional
from rdkit.Chem.rdchem import (
    ALLOW_CHARGE_SEPARATION as ALLOW_CHARGE_SEPARATION,
    ALLOW_INCOMPLETE_OCTETS as ALLOW_INCOMPLETE_OCTETS,
    AllProps as AllProps,
    AtomProps as AtomProps,
    BondProps as BondProps,
    CHI_OTHER as CHI_OTHER,
    CHI_TETRAHEDRAL_CCW as CHI_TETRAHEDRAL_CCW,
    CHI_TETRAHEDRAL_CW as CHI_TETRAHEDRAL_CW,
    CHI_UNSPECIFIED as CHI_UNSPECIFIED,
    COMPOSITE_AND as COMPOSITE_AND,
    COMPOSITE_OR as COMPOSITE_OR,
    COMPOSITE_XOR as COMPOSITE_XOR,
    ComputedProps as ComputedProps,
    CoordsAsDouble as CoordsAsDouble,
    KEKULE_ALL as KEKULE_ALL,
    MolProps as MolProps,
    NoConformers as NoConformers,
    NoProps as NoProps,
    PrivateProps as PrivateProps,
    QueryAtomData as QueryAtomData,
    STEREO_ABSOLUTE as STEREO_ABSOLUTE,
    STEREO_AND as STEREO_AND,
    STEREO_OR as STEREO_OR,
    UNCONSTRAINED_ANIONS as UNCONSTRAINED_ANIONS,
    UNCONSTRAINED_CATIONS as UNCONSTRAINED_CATIONS,
    Atom as Atom,
    AtomKekulizeException as AtomKekulizeException,
    AtomMonomerInfo as AtomMonomerInfo,
    AtomMonomerType as AtomMonomerType,
    AtomPDBResidueInfo as AtomPDBResidueInfo,
    AtomSanitizeException as AtomSanitizeException,
    AtomValenceException as AtomValenceException,
    Bond as Bond,
    BondDir as BondDir,
    BondStereo as BondStereo,
    BondType as BondType,
    ChiralType as ChiralType,
    CompositeQueryType as CompositeQueryType,
    Conformer as Conformer,
    EditableMol as EditableMol,
    FixedMolSizeMolBundle as FixedMolSizeMolBundle,
    HybridizationType as HybridizationType,
    KekulizeException as KekulizeException,
    Mol as Mol,
    MolBundle as MolBundle,
    MolSanitizeException as MolSanitizeException,
    PeriodicTable as PeriodicTable,
    PropertyPickleOptions as PropertyPickleOptions,
    QueryAtom as QueryAtom,
    QueryBond as QueryBond,
    RWMol as RWMol,
    ResonanceFlags as ResonanceFlags,
    ResonanceMolSupplier as ResonanceMolSupplier,
    ResonanceMolSupplierCallback as ResonanceMolSupplierCallback,
    RingInfo as RingInfo,
    StereoDescriptor as StereoDescriptor,
    StereoGroup as StereoGroup,
    StereoGroupType as StereoGroupType,
    StereoGroup_vect as StereoGroup_vect,
    StereoInfo as StereoInfo,
    StereoSpecified as StereoSpecified,
    StereoType as StereoType,
    SubstanceGroup as SubstanceGroup,
    SubstanceGroupAttach as SubstanceGroupAttach,
    SubstanceGroupCState as SubstanceGroupCState,
    SubstanceGroup_VECT as SubstanceGroup_VECT,
    SubstructMatchParameters as SubstructMatchParameters,
    AddMolSubstanceGroup as AddMolSubstanceGroup,
    ClearMolSubstanceGroups as ClearMolSubstanceGroups,
    CreateMolSubstanceGroup as CreateMolSubstanceGroup,
    CreateStereoGroup as CreateStereoGroup,
    GetAtomAlias as GetAtomAlias,
    GetAtomRLabel as GetAtomRLabel,
    GetAtomValue as GetAtomValue,
    GetDefaultPickleProperties as GetDefaultPickleProperties,
    GetMolSubstanceGroupWithIdx as GetMolSubstanceGroupWithIdx,
    GetMolSubstanceGroups as GetMolSubstanceGroups,
    GetPeriodicTable as GetPeriodicTable,
    GetSupplementalSmilesLabel as GetSupplementalSmilesLabel,
    SetAtomAlias as SetAtomAlias,
    SetAtomRLabel as SetAtomRLabel,
    SetAtomValue as SetAtomValue,
    SetDefaultPickleProperties as SetDefaultPickleProperties,
    SetSupplementalSmilesLabel as SetSupplementalSmilesLabel,
    tossit as tossit,
)

from rdkit.Chem.rdmolfiles import *
from rdkit.Chem.rdmolops import *
from rdkit.Chem.rdCIPLabeler import *
from rdkit.Chem.inchi import *
from rdkit.Chem.rdMolInterchange import *
from _typeshed import Incomplete
from rdkit import DataStructs as DataStructs, RDConfig as RDConfig, rdBase as rdBase
from rdkit.Chem import rdCoordGen as rdCoordGen, rdchem as rdchem
from rdkit.Geometry import rdGeometry as rdGeometry

_HasSubstructMatchStr: Incomplete
templDir: Incomplete

def QuickSmartsMatch(
    smi: str, sma: str, unique: bool = True, display: bool = False
) -> tuple[tuple[int, ...], ...]: ...
def CanonSmiles(smi: str, useChiral: int = 1) -> str: ...
def SupplierFromFilename(fileN: str, delim: str = "", **kwargs): ...
def FindMolChiralCenters(
    mol: Mol,
    force: bool = True,
    includeUnassigned: bool = False,
    includeCIP: bool = True,
    useLegacyImplementation: Optional[bool] = None,
) -> list[tuple[int, str]]:
    """
    >>> from rdkit import Chem
    >>> mol = Chem.MolFromSmiles('[C@H](Cl)(F)Br')
    >>> Chem.FindMolChiralCenters(mol)
    [(0, 'R')]
    >>> mol = Chem.MolFromSmiles('[C@@H](Cl)(F)Br')
    >>> Chem.FindMolChiralCenters(mol)
    [(0, 'S')]

    >>> Chem.FindMolChiralCenters(Chem.MolFromSmiles('CCC'))
    []

    By default unassigned stereo centers are not reported:

    >>> mol = Chem.MolFromSmiles('C[C@H](F)C(F)(Cl)Br')
    >>> Chem.FindMolChiralCenters(mol,force=True)
    [(1, 'S')]

    but this can be changed:

    >>> Chem.FindMolChiralCenters(mol,force=True,includeUnassigned=True)
    [(1, 'S'), (3, '?')]

    The handling of unassigned stereocenters for dependent stereochemistry is not correct
    using the legacy implementation:

    >>> Chem.FindMolChiralCenters(Chem.MolFromSmiles('C1CC(C)C(C)C(C)C1'),includeUnassigned=True)
    [(2, '?'), (6, '?')]
    >>> Chem.FindMolChiralCenters(Chem.MolFromSmiles('C1C[C@H](C)C(C)[C@H](C)C1'),includeUnassigned=True)
    [(2, 'S'), (4, '?'), (6, 'R')]

    But works with the new implementation:

    >>> Chem.FindMolChiralCenters(Chem.MolFromSmiles('C1CC(C)C(C)C(C)C1'),includeUnassigned=True, useLegacyImplementation=False)
    [(2, '?'), (4, '?'), (6, '?')]

    Note that the new implementation also gets the correct descriptors for para-stereochemistry:

    >>> Chem.FindMolChiralCenters(Chem.MolFromSmiles('C1C[C@H](C)[C@H](C)[C@H](C)C1'),useLegacyImplementation=False)
    [(2, 'S'), (4, 's'), (6, 'R')]

    With the new implementation, if you don't care about the CIP labels of stereocenters, you can save
    some time by disabling those:

    >>> Chem.FindMolChiralCenters(Chem.MolFromSmiles('C1C[C@H](C)[C@H](C)[C@H](C)C1'), includeCIP=False, useLegacyImplementation=False)
    [(2, 'Tet_CCW'), (4, 'Tet_CCW'), (6, 'Tet_CCW')]

    """

def WrapLogs() -> None: ...
def LogWarningMsg(msg: str) -> None: ...
def LogErrorMsg(msg: str) -> None: ...
def _test(): ...



from types import *
from _typeshed import Incomplete

TRACE: int

class HTMLPiddler:
    html: Incomplete
    start: Incomplete
    xLimits: Incomplete
    font: Incomplete
    color: Incomplete
    def __init__(
        self,
        html: str = ...,
        start=...,
        xLimits=...,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
    ) -> None: ...
    def renderOn(self, aPiddleCanvas) -> None: ...

DEMO_HTML: str

def demoPDF(html) -> None: ...
def demoPIL(html) -> None: ...
def demoTK(html) -> None: ...
def demoWX(html) -> None: ...
def demo(html=...) -> None: ...



from typing import Any, ClassVar

CENTROID: ClusterMethod
CLINK: ClusterMethod
GOWER: ClusterMethod
MCQUITTY: ClusterMethod
SLINK: ClusterMethod
UPGMA: ClusterMethod
WARD: ClusterMethod

class ClusterMethod:
    CENTROID: ClassVar[ClusterMethod] = ...
    CLINK: ClassVar[ClusterMethod] = ...
    GOWER: ClassVar[ClusterMethod] = ...
    MCQUITTY: ClassVar[ClusterMethod] = ...
    SLINK: ClassVar[ClusterMethod] = ...
    UPGMA: ClassVar[ClusterMethod] = ...
    WARD: ClassVar[ClusterMethod] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class HierarchicalClusterPicker:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def Cluster(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Pick(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class LeaderPicker:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def LazyBitVectorPick(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def LazyPick(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MaxMinPicker:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def LazyBitVectorPick(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def LazyBitVectorPickWithThreshold(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def LazyPick(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def LazyPickWithThreshold(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Pick(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...



from rdkit.Chem.rdfragcatalog import *
from _typeshed import Incomplete
from rdkit import Chem as Chem

def message(msg, dest=...) -> None: ...

class BitGainsInfo:
    id: int
    description: str
    gain: float
    nPerClass: Incomplete

def ProcessGainsFile(
    fileName, nToDo: int = ..., delim: str = ..., haveDescriptions: int = ...
): ...
def BuildAdjacencyList(
    catalog, bits, limitInclusion: int = ..., orderLevels: int = ...
): ...
def GetMolsMatchingBit(mols, bit, fps): ...



from _typeshed import Incomplete

def _ConvertModelPerformance(perf, modelPerf) -> None: ...
def PackageToXml(
    pkg,
    summary: str = ...,
    trainingDataId: str = ...,
    dataPerformance=...,
    recommendedThreshold: Incomplete | None = ...,
    classDescriptions: Incomplete | None = ...,
    modelType: Incomplete | None = ...,
    modelOrganism: Incomplete | None = ...,
): ...



from rdkit.sping.pid import *
from . import pdfgen as pdfgen, pdfgeom as pdfgeom, pdfmetrics as pdfmetrics
from _typeshed import Incomplete
from math import ceil as ceil
from rdkit.sping import pagesizes as pagesizes

DEFAULT_PAGE_SIZE: Incomplete
font_face_map: Incomplete
ps_font_map: Incomplete

class PDFCanvas(Canvas):
    pdf: Incomplete
    pagesize: Incomplete
    filename: Incomplete
    drawingsize: Incomplete
    pageTransitionString: str
    pageNumber: int
    def __init__(
        self, size: Incomplete | None = ..., name: str = ..., pagesize=...
    ) -> None: ...
    defaultLineColor: Incomplete
    defaultFillColor: Incomplete
    defaultLineWidth: int
    defaultFont: Incomplete
    def _resetDefaults(self) -> None: ...
    def showPage(self) -> None: ...
    def isInteractive(self): ...
    def canUpdate(self): ...
    def clear(self) -> None: ...
    def flush(self) -> None: ...
    def save(
        self, file: Incomplete | None = ..., format: Incomplete | None = ...
    ) -> None: ...
    def setInfoLine(self, s) -> None: ...
    def __setattr__(self, key, value) -> None: ...
    def _updateLineColor(self, color) -> None: ...
    def _updateFillColor(self, color) -> None: ...
    def _updateLineWidth(self, width) -> None: ...
    def _updateFont(self, font) -> None: ...
    def _findPostScriptFontName(self, font): ...
    def _escape(self, s): ...
    def resetDefaults(self) -> None: ...
    def stringWidth(self, s, font: Incomplete | None = ...): ...
    def fontHeight(self, font: Incomplete | None = ...): ...
    def fontAscent(self, font: Incomplete | None = ...): ...
    def fontDescent(self, font: Incomplete | None = ...): ...
    def _endPath(self, path, edgeColor, fillColor) -> None: ...
    def drawLine(
        self,
        x1,
        y1,
        x2,
        y2,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawLines(
        self,
        lineList,
        color: Incomplete | None = ...,
        width: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawString(
        self,
        s,
        x,
        y,
        font: Incomplete | None = ...,
        color: Incomplete | None = ...,
        angle: int = ...,
        **kwargs
    ) -> None: ...
    def drawCurve(
        self,
        x1,
        y1,
        x2,
        y2,
        x3,
        y3,
        x4,
        y4,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawRect(
        self,
        x1,
        y1,
        x2,
        y2,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawEllipse(
        self,
        x1,
        y1,
        x2,
        y2,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawArc(
        self,
        x1,
        y1,
        x2,
        y2,
        startAng: int = ...,
        extent: int = ...,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawPolygon(
        self,
        pointlist,
        edgeColor: Incomplete | None = ...,
        edgeWidth: Incomplete | None = ...,
        fillColor: Incomplete | None = ...,
        closed: int = ...,
        dash: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawImage(
        self,
        image,
        x1,
        y1,
        x2: Incomplete | None = ...,
        y2: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def drawLiteral(self, literal) -> None: ...

def test(): ...



from _typeshed import Incomplete
from rdkit.ML.Data import SplitData as SplitData
from rdkit.ML.Neural import Network as Network, Trainers as Trainers

def CrossValidate(net, testExamples, tolerance, appendExamples: int = ...): ...
def CrossValidationDriver(
    examples,
    attrs=...,
    nPossibleVals=...,
    holdOutFrac: float = ...,
    silent: int = ...,
    tolerance: float = ...,
    calcTotalError: int = ...,
    hiddenSizes: Incomplete | None = ...,
    **kwargs
): ...



from rdkit.Chem.FeatMaps import FeatMaps as FeatMaps

class MergeMethod:
    WeightedAverage: int
    Average: int
    UseLarger: int
    @classmethod
    def valid(cls, mergeMethod) -> None: ...

class MergeMetric:
    NoMerge: int
    Distance: int
    Overlap: int
    @classmethod
    def valid(cls, mergeMetric) -> None: ...

class DirMergeMode:
    NoMerge: int
    Sum: int
    @classmethod
    def valid(cls, dirMergeMode) -> None: ...

def __copyAll(res, fm1, fm2) -> None: ...
def GetFeatFeatDistMatrix(fm, mergeMetric, mergeTol, dirMergeMode, compatFunc): ...
def familiesMatch(f1, f2): ...
def feq(v1, v2, tol: float = ...): ...
def MergeFeatPoints(
    fm,
    mergeMetric=...,
    mergeTol: float = ...,
    dirMergeMode=...,
    mergeMethod=...,
    compatFunc=...,
): ...
def CombineFeatMaps(
    fm1, fm2, mergeMetric=..., mergeTol: float = ..., dirMergeMode=...
): ...



from _typeshed import Incomplete
from rdkit import Chem as Chem

log: Incomplete
MAX_STRUCTURES: int

class ResonanceEnumerator:
    kekule_all: Incomplete
    allow_incomplete_octets: Incomplete
    unconstrained_cations: Incomplete
    unconstrained_anions: Incomplete
    allow_charge_separation: Incomplete
    max_structures: Incomplete
    def __init__(
        self,
        kekule_all: bool = ...,
        allow_incomplete_octets: bool = ...,
        unconstrained_cations: bool = ...,
        unconstrained_anions: bool = ...,
        allow_charge_separation: bool = ...,
        max_structures=...,
    ) -> None: ...
    def __call__(self, mol): ...
    def enumerate(self, mol): ...

def enumerate_resonance_smiles(smiles): ...



from _typeshed import Incomplete

class CanvasBase:
    def addCanvasLine(
        self, p1, p2, color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def addCanvasText(self, text, pos, font, color=..., **kwargs) -> None: ...
    def addCanvasPolygon(self, ps, color=..., **kwargs) -> None: ...
    def addCanvasDashedWedge(
        self, p1, p2, p3, dash=..., color=..., color2: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def flush(self) -> None: ...
    def _getLinePoints(self, p1, p2, dash): ...



from types import *
from . import pdfmetrics as pdfmetrics, pdfutils as pdfutils
from .pdfgeom import bezierArc as bezierArc
from .pdfutils import LINEEND as LINEEND
from _typeshed import Incomplete
from math import ceil as ceil, cos as cos, pi as pi, sin as sin

StandardEnglishFonts: Incomplete
PDFError: str
AFMDIR: str
A4: Incomplete

class PDFDocument:
    objects: Incomplete
    objectPositions: Incomplete
    fonts: Incomplete
    fontMapping: Incomplete
    pages: Incomplete
    pagepositions: Incomplete
    PageCol: Incomplete
    fontdict: Incomplete
    info: Incomplete
    infopos: Incomplete
    def __init__(self) -> None: ...
    def add(self, key, obj): ...
    def getPosition(self, key): ...
    def setTitle(self, title) -> None: ...
    def setAuthor(self, author) -> None: ...
    def setSubject(self, subject) -> None: ...
    startxref: Incomplete
    def printXref(self) -> None: ...
    def writeXref(self, f) -> None: ...
    def printTrailer(self) -> None: ...
    def writeTrailer(self, f) -> None: ...
    def SaveToFile(self, filename) -> None: ...
    xref: Incomplete
    def SaveToFileObject(self, fileobj) -> None: ...
    def printPDF(self) -> None: ...
    def addPage(self, page) -> None: ...
    def hasFont(self, psfontname): ...
    def getInternalFontName(self, psfontname): ...
    def getAvailableFonts(self): ...

class OutputGrabber:
    oldoutput: Incomplete
    closed: int
    data: Incomplete
    def __init__(self) -> None: ...
    def write(self, x) -> None: ...
    def getData(self): ...
    def close(self) -> None: ...
    def __del__(self) -> None: ...

def testOutputGrabber() -> None: ...

class PDFObject:
    def save(self, file) -> None: ...
    def printPDF(self) -> None: ...

class PDFLiteral(PDFObject):
    text: Incomplete
    def __init__(self, text) -> None: ...
    def save(self, file) -> None: ...

class PDFCatalog(PDFObject):
    template: Incomplete
    def __init__(self) -> None: ...
    def save(self, file) -> None: ...

class PDFInfo(PDFObject):
    title: str
    author: str
    subject: str
    datestr: Incomplete
    def __init__(self) -> None: ...
    def save(self, file) -> None: ...

class PDFOutline(PDFObject):
    template: Incomplete
    def __init__(self) -> None: ...
    def save(self, file) -> None: ...

class PDFPageCollection(PDFObject):
    PageList: Incomplete
    def __init__(self) -> None: ...
    def save(self, file) -> None: ...

class PDFPage(PDFObject):
    drawables: Incomplete
    pagewidth: int
    pageheight: int
    stream: Incomplete
    hasImages: int
    pageTransitionString: str
    template: Incomplete
    def __init__(self) -> None: ...
    def setCompression(self, onoff: int = ...) -> None: ...
    def save(self, file) -> None: ...
    def clear(self) -> None: ...
    def setStream(self, data) -> None: ...

TestStream: str

class PDFStream(PDFObject):
    data: Incomplete
    compression: int
    def __init__(self) -> None: ...
    def setStream(self, data) -> None: ...
    def save(self, file) -> None: ...

class PDFImage(PDFObject):
    def save(self, file) -> None: ...

class PDFType1Font(PDFObject):
    fontname: Incomplete
    keyname: Incomplete
    template: Incomplete
    def __init__(self, key, font) -> None: ...
    def save(self, file) -> None: ...

def MakeType1Fonts(): ...
def MakeFontDictionary(startpos, count): ...



from _typeshed import Incomplete
from rdkit import Geometry as Geometry, RDConfig as RDConfig

_version: str
_usage: str
_welcomeMessage: Incomplete
logger: Incomplete
_featColors: Incomplete

def _getVectNormal(v, tol: float = ...): ...

_canonArrowhead: Incomplete

def _buildCanonArrowhead(headFrac, nSteps, aspect) -> None: ...

_globalArrowCGO: Incomplete
_globalSphereCGO: Incomplete
BEGIN: int
END: int
TRIANGLE_FAN: int
COLOR: int
VERTEX: int
NORMAL: int
SPHERE: int
CYLINDER: int
ALPHA: int

def _cgoArrowhead(
    viewer,
    tail,
    head,
    radius,
    color,
    label,
    headFrac: float = ...,
    nSteps: int = ...,
    aspect: float = ...,
) -> None: ...
def ShowArrow(
    viewer,
    tail,
    head,
    radius,
    color,
    label,
    transparency: int = ...,
    includeArrowhead: bool = ...,
) -> None: ...
def ShowMolFeats(
    mol,
    factory,
    viewer,
    radius: float = ...,
    confId: int = ...,
    showOnly: bool = ...,
    name: str = ...,
    transparency: float = ...,
    colors: Incomplete | None = ...,
    excludeTypes=...,
    useFeatDirs: bool = ...,
    featLabel: Incomplete | None = ...,
    dirLabel: Incomplete | None = ...,
    includeArrowheads: bool = ...,
    writeFeats: bool = ...,
    showMol: bool = ...,
    featMapFile: bool = ...,
) -> None: ...

parser: Incomplete



from _typeshed import Incomplete
from rdkit import Chem as Chem, RDConfig as RDConfig
from rdkit.Chem import AllChem as AllChem, rdChemReactions as rdChemReactions

def PreprocessReaction(
    reaction, funcGroupFilename: Incomplete | None = ..., propName: str = ...
): ...
def EnumerateReaction(
    reaction,
    bbLists,
    uniqueProductsOnly: bool = ...,
    funcGroupFilename=...,
    propName: str = ...,
): ...
def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from _typeshed import Incomplete
from rdkit import Geometry as Geometry
from rdkit.Chem import ChemicalFeatures as ChemicalFeatures
from rdkit.RDLogger import logger as logger

class Pharmacophore:
    _boundsMat: Incomplete
    _boundsMat2D: Incomplete
    def __init__(self, feats, initMats: bool = ...) -> None: ...
    _feats: Incomplete
    def _initializeFeats(self, feats) -> None: ...
    def _initializeMatrices(self) -> None: ...
    def getFeatures(self): ...
    def getFeature(self, i): ...
    def getUpperBound(self, i, j): ...
    def getLowerBound(self, i, j): ...
    def _checkBounds(self, i, j): ...
    def setUpperBound(self, i, j, val, checkBounds: bool = ...) -> None: ...
    def setLowerBound(self, i, j, val, checkBounds: bool = ...) -> None: ...
    def getUpperBound2D(self, i, j): ...
    def getLowerBound2D(self, i, j): ...
    def setUpperBound2D(self, i, j, val, checkBounds: bool = ...) -> None: ...
    def setLowerBound2D(self, i, j, val, checkBounds: bool = ...) -> None: ...
    def __str__(self): ...

class ExplicitPharmacophore:
    def __init__(
        self, feats: Incomplete | None = ..., radii: Incomplete | None = ...
    ) -> None: ...
    _feats: Incomplete
    _radii: Incomplete
    def _initializeFeats(self, feats, radii) -> None: ...
    def getFeatures(self): ...
    def getRadii(self): ...
    def getFeature(self, i): ...
    def getRadius(self, i): ...
    def setRadius(self, i, rad) -> None: ...
    def initFromString(self, text) -> None: ...
    def initFromFile(self, inF) -> None: ...
    def initFromLines(self, lines) -> None: ...
    def __str__(self): ...



from _typeshed import Incomplete
from rdkit.VLib.Node import VLibNode as VLibNode

class OutputNode(VLibNode):
    _dest: Incomplete
    _func: Incomplete
    def __init__(
        self, dest: Incomplete | None = ..., strFunc: Incomplete | None = ..., **kwargs
    ) -> None: ...
    def next(self): ...

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from typing import Any

def CanonicalizeConformer(*args, **kwargs) -> Any: ...
def CanonicalizeMol(RDKit) -> Any: ...
def ComputeCanonicalTransform(*args, **kwargs) -> Any: ...
def ComputeCentroid(RDKit) -> Any: ...
def ComputePrincipalAxesAndMoments(*args, **kwargs) -> Any: ...
def ComputePrincipalAxesAndMomentsFromGyrationMatrix(*args, **kwargs) -> Any: ...
def GetAngleDeg(*args, **kwargs) -> Any: ...
def GetAngleRad(*args, **kwargs) -> Any: ...
def GetBondLength(*args, **kwargs) -> Any: ...
def GetDihedralDeg(*args, **kwargs) -> Any: ...
def GetDihedralRad(*args, **kwargs) -> Any: ...
def SetAngleDeg(*args, **kwargs) -> Any: ...
def SetAngleRad(*args, **kwargs) -> Any: ...
def SetBondLength(*args, **kwargs) -> Any: ...
def SetDihedralDeg(*args, **kwargs) -> Any: ...
def SetDihedralRad(*args, **kwargs) -> Any: ...
def TransformConformer(RDKit, boost) -> Any: ...



from typing import Any

class EHTResults:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetAtomicCharges(cls, RDKit) -> Any: ...
    @classmethod
    def GetHamiltonian(cls, RDKit) -> Any: ...
    @classmethod
    def GetOrbitalEnergies(cls, RDKit) -> Any: ...
    @classmethod
    def GetOverlapMatrix(cls, RDKit) -> Any: ...
    @classmethod
    def GetReducedChargeMatrix(cls, RDKit) -> Any: ...
    @classmethod
    def GetReducedOverlapPopulationMatrix(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def fermiEnergy(self) -> Any: ...
    @property
    def numElectrons(self) -> Any: ...
    @property
    def numOrbitals(self) -> Any: ...
    @property
    def totalEnergy(self) -> Any: ...

def RunMol(RDKit) -> Any: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem.Suppliers.MolSupplier import MolSupplier as MolSupplier

def warning(msg, dest=...) -> None: ...

class DbMolSupplier(MolSupplier):
    _data: Incomplete
    _colNames: Incomplete
    molCol: int
    transformFunc: Incomplete
    nameCol: Incomplete
    molFmt: Incomplete
    _numProcessed: int
    def __init__(
        self,
        dbResults,
        molColumnFormats=...,
        nameCol: str = ...,
        transformFunc: Incomplete | None = ...,
        **kwargs
    ) -> None: ...
    def GetColumnNames(self): ...
    def _BuildMol(self, data): ...

class ForwardDbMolSupplier(DbMolSupplier):
    def __init__(self, dbResults, **kwargs) -> None: ...
    _dataIter: Incomplete
    def Reset(self) -> None: ...
    def NextMol(self): ...

class RandomAccessDbMolSupplier(DbMolSupplier):
    _pos: int
    def __init__(self, dbResults, **kwargs) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, idx): ...
    def Reset(self) -> None: ...
    def NextMol(self): ...



from rdkit import Geometry as Geometry
from rdkit.Chem.Subshape import SubshapeObjects as SubshapeObjects

def ComputeGridIndices(shapeGrid, winRad): ...
def ComputeShapeGridCentroid(pt, shapeGrid, winRad): ...
def FindTerminalPtsFromShape(shape, winRad, fraction, maxGridVal: int = ...): ...
def FindTerminalPtsFromConformer(conf, winRad, nbrCount): ...
def FindGridPointBetweenPoints(pt1, pt2, shapeGrid, winRad): ...
def ClusterTerminalPts(pts, winRad, scale): ...
def GetMoreTerminalPoints(
    shape, pts, winRad, maxGridVal, targetNumber: int = ...
) -> None: ...
def FindFarthestGridPoint(shape, loc, winRad, maxGridVal): ...
def ExpandTerminalPts(
    shape, pts, winRad, maxGridVal: float = ..., targetNumPts: int = ...
) -> None: ...
def AppendSkeletonPoints(
    shapeGrid,
    termPts,
    winRad,
    stepDist,
    maxGridVal: int = ...,
    maxDistC: float = ...,
    distTol: float = ...,
    symFactor: float = ...,
    verbose: bool = ...,
): ...
def CalculateDirectionsAtPoint(pt, shapeGrid, winRad) -> None: ...
def AssignMolFeatsToPoints(pts, mol, featFactory, winRad) -> None: ...



from typing import Any, ClassVar

from typing import overload

class AllowedAtomsValidation:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def validate(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class ChargeCorrection:
    __instance_size__: ClassVar[int] = ...
    Charge: Any
    Name: Any
    Smarts: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class CleanupParameters:
    __instance_size__: ClassVar[int] = ...
    acidbaseFile: Any
    doCanonical: Any
    fragmentFile: Any
    largestFragmentChooserCountHeavyAtomsOnly: Any
    largestFragmentChooserUseAtomCount: Any
    maxRestarts: Any
    maxTautomers: Any
    maxTransforms: Any
    normalizationsFile: Any
    preferOrganic: Any
    tautomerReassignStereo: Any
    tautomerRemoveBondStereo: Any
    tautomerRemoveIsotopicHs: Any
    tautomerRemoveSp3Stereo: Any
    tautomerTransformsFile: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class DisallowedAtomsValidation:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def validate(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FragmentRemover:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def remove(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FragmentValidation(MolVSValidations):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def run(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class IsotopeValidation(MolVSValidations):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def run(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class LargestFragmentChooser:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def choose(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MetalDisconnector:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def Disconnect(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetMetalNof(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetMetalNon(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def MetalNof(self) -> Any: ...
    @property
    def MetalNon(self) -> Any: ...

class MolVSValidation:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def validate(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MolVSValidations:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def run(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class NeutralValidation(MolVSValidations):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def run(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class NoAtomValidation(MolVSValidations):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def run(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class Normalizer:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def normalize(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class RDKitValidation:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def validate(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class Reionizer:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def reionize(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SmilesTautomerMap:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def items(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def keys(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def values(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class Tautomer:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def kekulized(self) -> Any: ...
    @property
    def tautomer(self) -> Any: ...

class TautomerEnumerator:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, RDKit): ...
    @classmethod
    def Canonicalize(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def Enumerate(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetCallback(cls, RDKit) -> Any: ...
    @classmethod
    def GetMaxTautomers(cls, RDKit) -> Any: ...
    @classmethod
    def GetMaxTransforms(cls, RDKit) -> Any: ...
    @classmethod
    def GetReassignStereo(cls, RDKit) -> Any: ...
    @classmethod
    def GetRemoveBondStereo(cls, RDKit) -> Any: ...
    @classmethod
    def GetRemoveSp3Stereo(cls, RDKit) -> Any: ...
    @classmethod
    def PickCanonical(cls, RDKit, boost) -> Any: ...
    def ScoreTautomer(self, *args, **kwargs) -> Any: ...
    @classmethod
    def SetCallback(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetMaxTautomers(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetMaxTransforms(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetReassignStereo(cls, RDKit, bool) -> Any: ...
    @classmethod
    def SetRemoveBondStereo(cls, RDKit, bool) -> Any: ...
    @classmethod
    def SetRemoveSp3Stereo(cls, RDKit, bool) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def tautomerScoreVersion(self) -> Any: ...

class TautomerEnumeratorCallback:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __call__(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class TautomerEnumeratorResult:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __call__(cls, Any) -> Any: ...
    @classmethod
    def __getitem__(cls, Any, int) -> Any: ...
    @classmethod
    def __iter__(cls, boost) -> Any: ...
    @classmethod
    def __len__(cls, Any) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def modifiedAtoms(self) -> Any: ...
    @property
    def modifiedBonds(self) -> Any: ...
    @property
    def smiles(self) -> Any: ...
    @property
    def smilesTautomerMap(self) -> Any: ...
    @property
    def status(self) -> Any: ...
    @property
    def tautomers(self) -> Any: ...

class TautomerEnumeratorStatus:
    Canceled: ClassVar[TautomerEnumeratorStatus] = ...
    Completed: ClassVar[TautomerEnumeratorStatus] = ...
    MaxTautomersReached: ClassVar[TautomerEnumeratorStatus] = ...
    MaxTransformsReached: ClassVar[TautomerEnumeratorStatus] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class Uncharger:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def uncharge(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class map_indexing_suite_SmilesTautomerMap_entry:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def data(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def key(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def CHARGE_CORRECTIONS() -> Any: ...
def CanonicalTautomer(RDKit) -> Any: ...
def ChargeParent(RDKit) -> Any: ...
def Cleanup(RDKit) -> Any: ...
def FragmentParent(RDKit) -> Any: ...
def FragmentRemoverFromData(*args, **kwargs) -> Any: ...
def GetV1TautomerEnumerator() -> Any: ...
def IsotopeParent(RDKit) -> Any: ...
def Normalize(RDKit) -> Any: ...
def NormalizerFromData(*args, **kwargs) -> Any: ...
def NormalizerFromParams(RDKit) -> Any: ...
def Reionize(RDKit) -> Any: ...
def ReionizerFromData(*args, **kwargs) -> Any: ...
def RemoveFragments(RDKit) -> Any: ...
def StandardizeSmiles(*args, **kwargs) -> Any: ...
def StereoParent(RDKit) -> Any: ...
def SuperParent(RDKit) -> Any: ...
def TautomerParent(RDKit) -> Any: ...
def UpdateParamsFromJSON(*args, **kwargs) -> Any: ...
def ValidateSmiles(*args, **kwargs) -> Any: ...



from typing import Any, ClassVar

APolar: SASAClass
LeeRichards: SASAAlgorithm
NACCESS: SASAClassifier
OONS: SASAClassifier
Polar: SASAClass
Protor: SASAClassifier
ShrakeRupley: SASAAlgorithm
Unclassified: SASAClass

class SASAAlgorithm:
    LeeRichards: ClassVar[SASAAlgorithm] = ...
    ShrakeRupley: ClassVar[SASAAlgorithm] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class SASAClass:
    APolar: ClassVar[SASAClass] = ...
    Polar: ClassVar[SASAClass] = ...
    Unclassified: ClassVar[SASAClass] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class SASAClassifier:
    NACCESS: ClassVar[SASAClassifier] = ...
    OONS: ClassVar[SASAClassifier] = ...
    Protor: ClassVar[SASAClassifier] = ...
    names: ClassVar[dict] = ...
    values: ClassVar[dict] = ...
    __slots__: ClassVar[tuple] = ...

class SASAOpts:
    __instance_size__: ClassVar[int] = ...
    algorithm: Any
    classifier: Any
    probeRadius: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

def CalcSASA(RDKit, boost) -> Any: ...
def MakeFreeSasaAPolarAtomQuery() -> Any: ...
def MakeFreeSasaPolarAtomQuery() -> Any: ...
def classifyAtoms(RDKit) -> Any: ...



from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.VLib.Transform import TransformNode as TransformNode

class SmartsRemover(TransformNode):
    _wholeFragments: Incomplete
    def __init__(self, patterns=..., wholeFragments: int = ..., **kwargs) -> None: ...
    _patterns: Incomplete
    def _initPatterns(self, patterns) -> None: ...
    def transform(self, cmpd): ...

biggerTest: str
__test__: Incomplete

def _runDoctests(verbose: Incomplete | None = ...) -> None: ...



from .utils import memoized_property as memoized_property
from _typeshed import Incomplete
from rdkit import Chem as Chem
from rdkit.Chem import rdMolDescriptors as rdMolDescriptors

log: Incomplete

class FragmentPattern:
    name: Incomplete
    smarts_str: Incomplete
    def __init__(self, name, smarts) -> None: ...
    def smarts(self): ...
    def __repr__(self): ...
    def __str__(self): ...

REMOVE_FRAGMENTS: Incomplete
LEAVE_LAST: bool
PREFER_ORGANIC: bool

def is_organic(fragment): ...

class FragmentRemover:
    fragments: Incomplete
    leave_last: Incomplete
    def __init__(self, fragments=..., leave_last=...) -> None: ...
    def __call__(self, mol): ...
    def remove(self, mol): ...

class LargestFragmentChooser:
    prefer_organic: Incomplete
    def __init__(self, prefer_organic=...) -> None: ...
    def __call__(self, mol): ...
    def choose(self, mol): ...



from typing import Any, ClassVar

class ExclusionList(FilterMatcherBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddPattern(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetExclusionPatterns(cls, RDKit, boost) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FilterCatalog:
    __instance_size__: ClassVar[int] = ...
    __safe_for_unpickling__: ClassVar[bool] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddEntry(cls, RDKit) -> Any: ...
    @classmethod
    def GetEntry(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetEntryWithIdx(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def GetFilterMatches(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetFirstMatch(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetMatches(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetNumEntries(cls, RDKit) -> Any: ...
    @classmethod
    def HasMatch(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def RemoveEntry(cls, RDKit, boost) -> Any: ...
    @classmethod
    def Serialize(cls, RDKit) -> Any: ...
    @classmethod
    def __getinitargs__(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FilterCatalogEntry:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def ClearProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetDescription(cls, RDKit) -> Any: ...
    @classmethod
    def GetFilterMatches(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetPropList(cls, RDKit) -> Any: ...
    @classmethod
    def HasFilterMatch(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def IsValid(cls, RDKit) -> Any: ...
    @classmethod
    def Serialize(cls, RDKit) -> Any: ...
    @classmethod
    def SetDescription(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetProp(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FilterCatalogEntryList:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class FilterCatalogListOfEntryList:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class FilterCatalogParams:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddCatalog(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FilterHierarchyMatcher(FilterMatcherBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def AddChild(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def SetPattern(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class FilterMatch:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @property
    def atomPairs(self) -> Any: ...
    @property
    def filterMatch(self) -> Any: ...

class FilterMatcherBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetMatches(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def GetName(cls, RDKit) -> Any: ...
    @classmethod
    def HasMatch(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def IsValid(cls, RDKit) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class IntPair:
    __instance_size__: ClassVar[int] = ...
    query: Any
    target: Any
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class MatchTypeVect:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class MolList:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

class PythonFilterMatcher(FilterMatcherBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class SmartsMatcher(FilterMatcherBase):
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def GetMaxCount(cls, RDKit) -> Any: ...
    @classmethod
    def GetMinCount(cls, RDKit) -> Any: ...
    @classmethod
    def GetPattern(cls, RDKit) -> Any: ...
    @classmethod
    def IsValid(cls, RDKit) -> Any: ...
    @classmethod
    def SetMaxCount(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetMinCount(cls, RDKit, unsignedint) -> Any: ...
    @classmethod
    def SetPattern(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...

class VectFilterMatch:
    __instance_size__: ClassVar[int] = ...
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    @classmethod
    def append(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def extend(cls, *args, **kwargs) -> Any: ...
    @classmethod
    def __contains__(cls, other) -> Any: ...
    @classmethod
    def __delitem__(cls, other) -> Any: ...
    @classmethod
    def __getitem__(cls, index) -> Any: ...
    @classmethod
    def __iter__(cls, boost, std) -> Any: ...
    @classmethod
    def __len__(cls) -> Any: ...
    @classmethod
    def __reduce__(cls) -> Any: ...
    @classmethod
    def __setitem__(cls, index, object) -> Any: ...

def FilterCatalogCanSerialize(*args, **kwargs) -> Any: ...
def GetFlattenedFunctionalGroupHierarchy(*args, **kwargs) -> Any: ...
def GetFunctionalGroupHierarchy() -> Any: ...
def RunFilterCatalog(*args, **kwargs) -> Any: ...
